/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.gridschool.unicore.plugins.povray;

import java.awt.Font;
import java.io.File;

import com.pallas.unicore.client.util.Defaults;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * Define, load and save plugin default values
 * 
 * @author Ralf Ratering
 * @version $Id: PovrayDefaults.java,v 1.1 2004/05/25 14:58:51 rmenday Exp $
 */
public class PovrayDefaults extends Defaults {

	/**
	 * String constants defining property keys
	 */
	public final static String INPUTDIRECTORY = "plugin.povray.inputdirectory",
			OUTPUTDIRECTORY = "plugin.povray.outputdirectory",
			WIDTH = "plugin.povray.width", HEIGHT = "plugin.povray.height",
			ANTIALIASING = "plugin.povray.antialiasing",
			EDITORFONTNAME = "plugin.povray.editor.fontname",
			EDITORFONTSIZE = "plugin.povray.editor.fontsize",
			EDITORLINEWRAP = "plugin.povray.editor.linewrap";

	/**
	 * Fill properties with default values
	 */
	protected void genSystemDefaults() {
		put(INPUTDIRECTORY, ResourceManager.getJarDirectory() + File.separator
				+ "povray");
		put(OUTPUTDIRECTORY, userHome);
		put(WIDTH, "320");
		put(HEIGHT, "200");
		put(ANTIALIASING, "0.3");
		put(EDITORFONTNAME, "monospaced.plain");
		put(EDITORFONTSIZE, "12");
		put(EDITORLINEWRAP, "true");
	}

	public double getAntiAliasing() {
		if (get(ANTIALIASING) == null) {
			put(ANTIALIASING, "0.3");
		}
		return doubleValue(ANTIALIASING);
	}

	/**
	 * Get the editor font
	 * 
	 * @return The editor font
	 */
	public Font getEditorFont() {
		String fontName = (String) get(EDITORFONTNAME);
		int fontSize = intValue(EDITORFONTSIZE);
		return new Font(fontName, Font.PLAIN, fontSize);
	}

	public int getHeight() {
		if (get(HEIGHT) == null) {
			put(HEIGHT, "200");
		}
		return intValue(HEIGHT);
	}

	/**
	 * Get the script directory
	 * 
	 * @return script directory value
	 */
	public String getInputDirectory() {
		return getProperty(INPUTDIRECTORY);
	}

	/**
	 * Get the line wrap flag for editor
	 * 
	 * @return boolean line wrap value
	 */
	public boolean getLineWrap() {
		return booleanValue(EDITORLINEWRAP);
	}

	public String getOutputDirectory() {
		return getProperty(OUTPUTDIRECTORY);
	}

	public int getWidth() {
		if (get(WIDTH) == null) {
			put(WIDTH, "320");
		}
		return intValue(WIDTH);
	}

	/**
	 * Load script default values from file ${HOME}/.unicore/PovrayDefaults.txt
	 * 
	 * @return true if loading was successful, else false
	 */
	public boolean loadFromFile() {
		// look for ${HOME}/.unicoreXX/PovrayDefaults.txt
		return loadFromFile(unicoreDir + File.separator + "PovrayDefaults.txt");
	}

	public void setAntiAliasing(double aa) {
		put(ANTIALIASING, new Double(aa).toString());
	}

	/**
	 * Set the editor font
	 * 
	 * @param font
	 *            The new editor font
	 */
	public void setEditorFont(Font font) {
		put(EDITORFONTNAME, font.getFontName());
		put(EDITORFONTSIZE, new Integer(font.getSize()).toString());
	}

	public void setHeight(int height) {
		put(HEIGHT, new Integer(height).toString());
	}

	/**
	 * Set the script directory
	 * 
	 * @param s
	 *            The new script directory
	 */
	public void setInputDirectory(String s) {
		put(INPUTDIRECTORY, s);
	}

	/**
	 * Set the line wrap flag for editor
	 * 
	 * @param b
	 *            boolean line wrap flag
	 */
	public void setLineWrap(boolean b) {
		put(EDITORLINEWRAP, new Boolean(b).toString());
	}

	public void setOutputDirectory(String s) {
		put(OUTPUTDIRECTORY, s);
	}

	public void setWidth(int width) {
		put(WIDTH, new Integer(width).toString());
	}

	/**
	 * Try to write script default values to file
	 * ${HOME}/.unicore/PovrayDefaults.txt
	 * 
	 * @return true if saving was successful, else false
	 */
	public boolean writeToFile() {
		// always overwrite old ${HOME}/.unicore/PovrayDefaults.txt
		return writeToFile(unicoreDir + File.separator + "PovrayDefaults.txt");
	}
}