package org.gridschool.unicore.plugins.smallservice;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.unicore.Vsite;
import org.unicore.ajo.AbstractJob;
import org.unicore.outcome.AbstractActionStatus;
import org.unicore.outcome.Outcome;
import org.unicore.resources.SoftwareResource;

import com.pallas.unicore.client.dialogs.ShowLogDialog;
import com.pallas.unicore.client.panels.JPAPanel;
import com.pallas.unicore.client.trees.ContainerNode;
import com.pallas.unicore.client.util.ExtensionPlugable;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.container.TaskContainer;
import com.pallas.unicore.extensions.FileExport;
import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.requests.DeleteJob;
import com.pallas.unicore.requests.GetJobStatus;
import com.pallas.unicore.requests.GetOutcome;
import com.pallas.unicore.requests.GetSpooledFiles;
import com.pallas.unicore.resourcemanager.OutcomeEntry;
import com.pallas.unicore.resourcemanager.OutcomeManager;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.threadpool.IObserver;
import com.pallas.unicore.utility.UserMessages;

/**
 *  Example for small service plugin. This plugin adds a button to the vsite
 *  list in the job group panel. When the button is pushed the name of a
 *  software resource is requested. If the specified software resource is
 *  defined in the IDB at that site it will be executed and then standard out is
 *  fetched back to the client and displayed in a dialog box.
 *
 *@author     Ralf Ratering
 *@version    $Id: SmallServicePlugin.java,v 1.3.2.1 2004/05/03 08:04:17 rraterin Exp $
 */
public class SmallServicePlugin
	extends ExtensionPlugable
	implements IObserver {
	private final static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.plugins.smallservice");

	private JobContainer job;
	private String softwareName = "smallservice";
	private JButton startButton;
	private SmallServiceContainer task;

	/**
	 *  Get a short info message for the plugin
	 *
	 *@return    String with info message
	 */
	public String getPluginInfo() {
		return "UNICORE Small service plugin example\n"
			+ "Demonstrating the use of application resources\n"
			+ "and polling for output files.\n"
			+ "(C)2004 Intel GmbH";
	}

	/**
	 *  Gets the vsiteToolBarComponent attribute of the SmallServicePlugin
	 *  object
	 *
	 *@return    The vsiteToolBarComponent value
	 */
	public Component getVsiteToolBarComponent() {
		// return a new instance each time...
		startButton.setToolTipText("A small service plugin example");
		return startButton;
	}

	/**
	 *  This method implements the IObserver interface and will be executed when
	 *  the GetOutcome request that was sent from submitServiceJob returns.
	 *
	 *@param  theObserved  reference to GetOutcome request
	 *@param  changeCode   null in this case
	 */
	public void observableUpdate(Object theObserved, Object changeCode) {
		if (theObserved instanceof GetJobStatus) {
			AbstractActionStatus status = (AbstractActionStatus) changeCode;
			if (status == null) {
				logger.warning("No status available.");
				return;
			}

			logger.info("Current SmallService Job Status: " + status);

			// still executing?
			if (status.isEquivalent(AbstractActionStatus.NOT_DONE)) {
				logger.info(
					"SmallService job still executing, "
						+ " sending another GetJobStatus request...");
				try {
					Thread.sleep(2000);
				}
				catch (InterruptedException e) {
					logger.log(Level.SEVERE, "Unexpected exception.", e);
				}
				sendGetJobStatus();
				return;
			}

			// not successful?
			if (status.isEquivalent(AbstractActionStatus.NOT_SUCCESSFUL)) {
				UserMessages.error(
					"SmallService job not executed successfully.");
				return;
			}

			// SUCCESS! Now get the outcome and implicitly unspool outcome files
			logger.info("Sending GetOutcome...");
			sendGetOutcome();
		}
		else if (theObserved instanceof GetOutcome) {
			// when GetOutcome returned get files from spooling area
			logger.info("Sending GetSpooledFiles...");
			sendGetSpooledFiles();

			try {
				GetOutcome getOutcome = (GetOutcome) theObserved;
				// get back the job container from the request
				ContainerNode node = (ContainerNode) getOutcome.getNode();
				JobContainer job = (JobContainer) node.getActionContainer();
				// NEW: Get Vector of outcome entries from OutcomeManager
				// Since we do not use iterations, select first Vector element
				Vector outcomeEntries =
					OutcomeManager.getOutcomeEntries(task.getExecuteTask());
				OutcomeEntry outcomeEntry =
					(OutcomeEntry) outcomeEntries.firstElement();
				Outcome outcome = outcomeEntry.getOutcome();
				if (outcome == null) {
					throw new Exception("No outcome available.");
				}
				// NEW: get stdout file from OutcomeManager entry
				String stdout = readFile(outcomeEntry.getStdoutFile());
				String message =
					"Successfully fetched standard out:\n" + stdout;
				ShowLogDialog dialog =
					new ShowLogDialog(
						ResourceManager.getCurrentInstance(),
						message,
						"UNICORE: Standard Out");
				dialog.setLocationRelativeTo(dialog.getParent());
				dialog.setVisible(true);
			}
			catch (Exception e) {
				UserMessages.info(
					"Could not read stdout file.\nReason: " + e.getMessage());
			}

		}
		else if (theObserved instanceof GetSpooledFiles) {
			GetSpooledFiles getSpooledFiles = (GetSpooledFiles) theObserved;
			String message =
				"Successfully unspooled file "
					+ getSpooledFiles.getFileExport();
			ShowLogDialog dialog =
				new ShowLogDialog(
					ResourceManager.getCurrentInstance(),
					message,
					"UNICORE: Unspooling finished");
			dialog.setLocationRelativeTo(dialog.getParent());
			dialog.setVisible(true);

			logger.info("Sending Deleting Job...");
			DeleteJob deleteJob =
				new DeleteJob(new ContainerNode(job, new JPanel()));
			deleteJob.addObserver(this);
			deleteJob.start();
		}
		else if (theObserved instanceof DeleteJob) {
			DeleteJob deleteJob = (DeleteJob) theObserved;
			ContainerNode node = (ContainerNode) deleteJob.getNode();
			JobContainer job = (JobContainer) node.getActionContainer();
			String message = "Successfully deleted job " + job;
			ShowLogDialog dialog =
				new ShowLogDialog(
					ResourceManager.getCurrentInstance(),
					message,
					"UNICORE: Job Deleted");
			dialog.setLocationRelativeTo(dialog.getParent());
			dialog.setVisible(true);

		}
	}

	private String readFile(File file)
		throws IOException, FileNotFoundException {
		BufferedReader reader = new BufferedReader(new FileReader(file));
		String result = "";
		while (reader.ready()) {
			result += reader.readLine() + "\n";
		}
		return result;
	}

	private void sendGetJobStatus() {
		GetJobStatus getStatus =
			new GetJobStatus((AbstractJob) job.getActionGroup());
		getStatus.addObserver(this);
		getStatus.start();
	}

	private void sendGetOutcome() {
		GetOutcome go = new GetOutcome(new ContainerNode(job, null));
		go.addObserver(this);
		go.start();
	}

	private void sendGetSpooledFiles() {
		// read stdout from a file in a local tmp directory
		// Files holen
		ContainerNode node = new ContainerNode(job, null);
		Vector newNspaceTransfers = job.getNspaceExportTasks();
		for (int i = 0; i < newNspaceTransfers.size(); i++) {
			TaskContainer ac = (TaskContainer) newNspaceTransfers.elementAt(i);
			Vector spooledFiles = ac.getSpooledFiles();
			if (spooledFiles == null) {
				continue;
			}
			for (int j = 0; j < spooledFiles.size(); j++) {
				FileExport export = (FileExport) spooledFiles.elementAt(j);
				if (export.getDestinationName().equals("")
					&& !export.hasWildcards()) {
					export.setDestinationName(export.getSourceName());
				}
				File testFile = new File(export.getDestinationName());
				String absolutePath = testFile.getAbsolutePath();
				export.setDestinationName(absolutePath);

				GetSpooledFiles fetch = new GetSpooledFiles(node, job, export);
				logger.info("Unspooling: " + export);
				fetch.addObserver(this);
				fetch.start();
			}
		}
	}

	/**
	 *  Check, if a software resource with a specific name is available at a
	 *  site. For convenience, we do not look for versions here... If the
	 *  resource is available put is as "PreinstalledSoftware" into our
	 *  SmallServiceContainer.
	 *
	 *@param  vsite  site to look at
	 *@return        true, if resource is available
	 */
	private boolean softwareAvailable(Vsite vsite) {
		// get all resources from site
		NamedResourceSet siteResources = ResourceManager.getResourceSet(vsite);
		SoftwareResource serviceSoftware = null;
		// selecte only software resources
		Vector siteSoftwares = new Vector();
		siteResources.addSoftwareResources(siteSoftwares);
		// loop over software resources
		for (int i = 0; i < siteSoftwares.size(); i++) {
			if (siteSoftwares.elementAt(i) instanceof SoftwareResource) {
				SoftwareResource siteSoftware =
					(SoftwareResource) siteSoftwares.elementAt(i);
				if (siteSoftware.getName().equals(softwareName)) {
					// success!
					serviceSoftware = siteSoftware;
				}
			}
		}
		if (serviceSoftware == null) {
			return false;
		}
		else {
			// if software was available, put it into the SmallServiceContainer
			task.setPreinstalledSoftware(serviceSoftware);
			return true;
		}
	}

	/**
	 *  Called to inititialize the plugin
	 */
	public void startPlugin() {
		job = new JobContainer();
		task = new SmallServiceContainer(job);
		job.addTask(task);
		startButton = new JButton(new ServiceAction());
	}

	/**
	 *  Stop plugin
	 */
	public void stopPlugin() {
		// do nothing
	}

	/**
	 *  Submit the service job
	 *
	 *@param  vsite  Vsite where the job will be submitted
	 */
	private void submitServiceJob(Vsite vsite) {
		System.err.println("Submitting job to vsite " + vsite + " ...");

		job.setName(
			ResourceManager.getServicePrefix()
				+ "SmallServiceJob"
				+ ResourceManager.getNextObjectIdentifier());
		job.setVsite(vsite);
		job.setUser(ResourceManager.getUser(vsite));

		// submit the job
		job.run();
	}

	private class ServiceAction extends javax.swing.AbstractAction {
		public ServiceAction() {
			super("", ResourceManager.getIcon(ResourceManager.REFRESH));
		}

		public void actionPerformed(ActionEvent event) {
			// get selected job from jpa tree
			ContainerNode jobNode =
				getClient().getJPATree().getCurrentJobNode();
			if (jobNode == null) {
				return;
			}

			// apply current gui entries to job container
			JPAPanel panel = (JPAPanel) jobNode.getPanel();
			panel.applyValues();

			// get vsite from job container
			JobContainer jobContainer =
				(JobContainer) jobNode.getActionContainer();
			Vsite vsite = jobContainer.getVsite();

			if (vsite == null) {
				UserMessages.info("No virtual site selected.");
				return;
			}

			// ask for resource name
			softwareName =
				(String) JOptionPane.showInputDialog(
					getClient(),
					"Enter name of Software Resource to be executed.",
					"UNICORE: Select Software Resource",
					JOptionPane.QUESTION_MESSAGE,
					null,
					null,
					softwareName);

			if (softwareName == null) {
				return;
			}

			// if SoftwareResource is not available at the site get out.
			if (!softwareAvailable(vsite)) {
				UserMessages.error(
					"A SoftwareResource with name <"
						+ softwareName
						+ "> is not available at site "
						+ vsite);
				return;
			}

			// ask if we should really start now...
			if (JOptionPane
				.showConfirmDialog(
					getClient(),
					"Execute software <"
						+ softwareName
						+ "> at site "
						+ vsite
						+ "?",
					"UNICORE: Small Services",
					JOptionPane.YES_NO_OPTION,
					JOptionPane.QUESTION_MESSAGE)
				== JOptionPane.YES_OPTION) {
				// submit job...
				submitServiceJob(vsite);
				// ...and fetch back outcome
				sendGetJobStatus();
			}
		}
	}

}
