/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.controls;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Vector;
import java.util.logging.Logger;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JComboBox;
import javax.swing.JList;
import org.unicore.Vsite;
import org.unicore.resources.Storage;
import org.unicore.resources.USpace;
import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * Combo box which keeps track of an unselectable storage i.e. a storage which
 * was available at a different site but not at the current one. An unselectable
 * storage will be remembered as unselectableEntry and added (temporarely) to
 * the combo box model. A listener will remove it on-the-fly from the model if
 * the combo box is opened.
 * 
 * @author Ralf Ratering
 * @author Kirsten Foerster
 * @author Klaus-Dieter Oertel
 * @version $Id: StorageComboBox.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 *  
 */
public class StorageComboBox extends JComboBox {

	protected static Logger logger = Logger
			.getLogger("com.pallas.unicore.client.controls");

	private class ComboBoxListener implements ItemListener {

		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.DESELECTED) {
				return;
			}
			if (e.getStateChange() == ItemEvent.SELECTED) {
				Object item = e.getItem();
				if (unselectableEntry != null
						&& !unselectableEntry.equals(getSelectedItem())) {
					entries.remove(unselectableEntry);
					unselectableEntry = null;
				}
			}
		}
	}

	/**
	 * ComboBoxRenderer shows NOT_AVAILABLE if no Vsite has been selected up to
	 * now i.e. the model has only a null value.
	 */
	class ComboBoxRenderer extends DefaultListCellRenderer {

		private StorageComboBox comboBox;

		public ComboBoxRenderer(StorageComboBox comboBox) {
			super();
			this.comboBox = comboBox;
		}

		public Component getListCellRendererComponent(JList list, Object value,
				int index, boolean isSelected, boolean cellHasFocus) {
			if (value == null) {
				value = NOT_AVAILABLE;
				//				list.setForeground(Color.GREEN);
				StorageComboBox.this.setForeground(Color.GREEN);
			}
			else if (unselectableEntry != null) {
				//				list.setForeground(Color.RED);
				StorageComboBox.this.setForeground(Color.RED);
			}
			else {
				//				list.setForeground(Color.BLACK);
				StorageComboBox.this.setForeground(Color.BLACK);
			}
			return super.getListCellRendererComponent(list, value, index,
					isSelected, cellHasFocus);
		}
	}
	public final static String NOT_AVAILABLE = "Not Available";
	private Vector entries;
	private DefaultComboBoxModel model;
	private Object unselectableEntry = null;

	public StorageComboBox() {
		entries = new Vector();
		model = new DefaultComboBoxModel(entries);
		setModel(model);
		this.addItemListener(new ComboBoxListener());
		this.setRenderer(new ComboBoxRenderer(this));
	}

	public boolean browsableItem() {
		Object selectedItem = getSelectedItem();
		if (selectedItem == null || !(selectedItem instanceof ResourceEntry)) {
			return false;
		}
		ResourceEntry entry = (ResourceEntry) selectedItem;
		if (entry.equals(unselectableEntry)
				|| entry.getResource() instanceof USpace) {
			return false;
		}
		else {
			return true;
		}
	}

	private ResourceEntry findResourceEntry(Storage storage) {
		ResourceEntry toBeFound = new ResourceEntry(storage);
		for (int i = 0; i < entries.size(); i++) {
			if (!(entries.elementAt(i) instanceof ResourceEntry)) {
				continue;
			}
			ResourceEntry entry = (ResourceEntry) entries.elementAt(i);
			if (entry.equals(toBeFound)) {
				return entry;
			}
		}
		return null;
	}

	public Object getSelectedItem() {
		Object item = super.getSelectedItem();
		return item;
	}

	public Object getSelectedStorage() {
		return getStorageAt(getSelectedIndex());
	}

	public Object getStorageAt(int i) {
		Object item = this.getItemAt(i);
		if (item instanceof ResourceEntry) {
			ResourceEntry entry = (ResourceEntry) item;
			return entry.getResource();
		}
		return item;
	}

	public void removeStorage(Storage storage) {
		removeItem(findResourceEntry(storage));
	}

	public void setSelectedStorage(Storage storage) {
		if (storage == null) {
			logger.info("setSelectedStorage: returning null");
			return;
		}
		if (unselectableEntry != null) {
			entries.removeElement(unselectableEntry);
			unselectableEntry = null;
		}
		ResourceEntry entry = findResourceEntry(storage);
		// storage not found ==> add this unselectable entry to the model by
		// replacing the previous one (if this exists)
		if (entry == null) {
			unselectableEntry = new ResourceEntry(storage);
			entries.addElement(unselectableEntry);
			entry = findResourceEntry(storage);
		}
		super.setSelectedItem(entry);
	}

	public void update(Vsite vsite) {
		ResourceEntry selectedItem = (ResourceEntry) getSelectedItem();
		// get site resources
		NamedResourceSet siteResources = ResourceManager.getResourceSet(vsite);
		// put site storages into storages vector
		Vector storages = new Vector();
		siteResources.addStorageResources(storages);
		// clear previous entries
		entries.removeAllElements();
		// build vector with resource entries
		entries.clear();
		for (int i = 0; i < storages.size(); i++) {
			ResourceEntry entry = new ResourceEntry((Storage) storages
					.elementAt(i));
			entries.addElement(entry);
		}
		this.setModel(new DefaultComboBoxModel(entries));
		// this.updateUI();
		// try to re-select item
		unselectableEntry = null;
		if (selectedItem != null) {
			logger.finest("Trying to select " + selectedItem);
			setSelectedStorage((Storage) selectedItem.getResource());
		}
		else if (entries.size() > 0) {
			USpace uspace = ResourceManager.getResourceSet(vsite).getUspace();
			logger.finest("Trying to select USpace " + uspace);
			setSelectedStorage(ResourceManager.getResourceSet(vsite)
					.getUspace());
		}
		logger.finest("Selected " + getSelectedItem());
	}
}