/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.graphs;

import java.util.Vector;
import com.pallas.unicore.utility.VectorSorter;

/**
 * Directed acyclic graph
 * 
 * @author Ralf Ratering
 * @version $Id: DAG.java,v 1.1 2004/05/25 14:58:45 rmenday Exp $
 */
public class DAG extends Graph {

	private int maxDepth;

	/**
	 * add an edge
	 * 
	 * @param edge
	 *            new edge to be added
	 * @exception Exception
	 *                thrown if cycle will be added
	 */
	public void addEdge(Edge edge) throws Exception {
		if (causesCycle(edge)) {
			throw new Exception(
					"Cannot add "
							+ edge
							+ ", because it would cause a cycle in the directed acyclic graph.");
		}
		if (!edges.contains(edge)) {
			edges.add(edge);
			edge.getParentVertex().addEdge(edge);
			edge.getChildVertex().addEdge(edge);
			edge.setColor(color);
			edge.setSelectionColor(selectionColor);
		}
	}

	public boolean causesCycle(Edge edge) {
		return pathExists((DAGVertex) edge.getChildVertex(), (DAGVertex) edge
				.getParentVertex());
	}

	/**
	 * Compute depth of a single vertex
	 * 
	 * @param vertex
	 *            vertex to be computed
	 */
	private void computeDepth(DAGVertex vertex) {
		Vector children = vertex.getChildren();
		for (int i = 0; i < children.size(); i++) {
			DAGVertex child = (DAGVertex) children.get(i);
			int newDepth = vertex.getDepth() + 1;
			// only increase depths
			if (newDepth > child.getDepth()) {
				child.setDepth(vertex.getDepth() + 1);
				computeDepth(child);
			}
		}
	}

	/**
	 * Compute depth for vertices
	 */
	public void computeDepths() {
		// reset depths
		for (int i = 0; i < getNumberOfVertices(); i++) {
			((DAGVertex) getVertex(i)).setDepth(0);
		}
		// loop over vertices and start recursion for root vertices (depth==0)
		for (int i = 0; i < getNumberOfVertices(); i++) {
			DAGVertex vertex = (DAGVertex) getVertex(i);
			if (vertex.isRoot()) {
				computeDepth(vertex);
			}
		}
		computeMaxDepth();
	}

	/**
	 * Find out maximum depth in graph
	 */
	private void computeMaxDepth() {
		maxDepth = 0;
		// loop over vertices and start recursion for root vertices (depth==0)
		for (int i = 0; i < getNumberOfVertices(); i++) {
			DAGVertex vertex = (DAGVertex) getVertex(i);
			if (vertex.getDepth() > maxDepth) {
				maxDepth = vertex.getDepth();
			}
		}
	}

	public boolean contains(Edge edge) {
		for (int i = 0; i < edges.size(); i++) {
			if (edge.equals(edges.elementAt(i))) {
				return true;
			}
		}
		return false;
	}

	public int getDepth() {
		return maxDepth;
	}

	/**
	 * Get all nodes at a certain depth
	 * 
	 * @param depth
	 *            depth level for nodes
	 * @return vector of vertices
	 */
	public Vector getNodesByDepth(int depth) {
		Vector result = new Vector();
		for (int i = 0; i < getNumberOfVertices(); i++) {
			DAGVertex vertex = (DAGVertex) getVertex(i);
			if (vertex.getDepth() == depth) {
				result.add(vertex);
			}
		}
		return result;
	}

	/**
	 * Get a Vector containing all possile successors of the DAG vertex
	 * 
	 * @param vertex
	 *            vertex to get successors from
	 * @return a Vector containing all possible successors
	 */
	public Vector getPossibleSuccs(Vertex vertex) {
		return vertices;
	}

	public Vector getSortedVertices(boolean ascending) {
		// depths will be used for sorting
		computeDepths();
		// Vector preSorted = VectorSorter.sort(vertices, ascending, true);
		return VectorSorter.sort(vertices, ascending);
	}

	/**
	 * Check if a path exists between two vertices
	 * 
	 * @param start
	 *            start vertex
	 * @param end
	 *            end vertex
	 * @return true, if path exists
	 */
	public boolean pathExists(DAGVertex start, DAGVertex end) {
		if (start == end) {
			return true;
		}
		Vector children = start.getChildren();
		for (int i = 0; i < children.size(); i++) {
			if (pathExists((DAGVertex) children.elementAt(i), end)) {
				return true;
			}
		}
		return false;
	}
}