/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.graphs;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import javax.swing.JPanel;
import com.pallas.unicore.client.graphics.Arrow;
import com.pallas.unicore.client.graphics.Drawable;
import com.pallas.unicore.client.graphics.Selectable;

/**
 * Edge in a DAG. For cyclic graphs directions may be ignored.
 * 
 * @author Ralf Ratering
 * @version $Id: Edge.java,v 1.1 2004/05/25 14:58:45 rmenday Exp $
 */
public class Edge implements Selectable, Drawable {

	private Arrow arrow = new Arrow(new Point2D.Double(0.0, 0.0),
			new Point2D.Double(0.0, 0.0));
	private Vertex childVertex, parentVertex;
	private Color color = Color.black, selectionColor = Color.red;
	private Line2D line = new Line2D.Double(0.0, 0.0, 0.0, 0.0);
	private String name;
	private boolean selected;

	/**
	 * Constructor for the Edge object
	 * 
	 * @param name
	 *            name for edge
	 * @param parentVertex
	 *            start vertex
	 * @param childVertex
	 *            end vertex
	 */
	public Edge(String name, Vertex parentVertex, Vertex childVertex) {
		this.name = name;
		this.parentVertex = parentVertex;
		this.childVertex = childVertex;
		setSelected(false);
	}

	/**
	 * Constructor for the Edge object
	 * 
	 * @param parentVertex
	 *            start vertex
	 * @param childVertex
	 *            end vertex
	 */
	public Edge(Vertex parentVertex, Vertex childVertex) {
		this("", parentVertex, childVertex);
	}

	/**
	 * Test if a vertex belongs to this edge
	 * 
	 * @param vertex
	 *            vertex to be tested
	 * @return true if vertex belongs to edge
	 */
	public boolean contains(Vertex vertex) {
		if (vertex.equals(childVertex) || vertex.equals(parentVertex)) {
			return true;
		}
		else {
			return false;
		}
	}

	public void draw(Graphics g, JPanel panel) {
		update();
		Graphics2D g2 = (Graphics2D) g;
		if (selected) {
			g2.setColor(selectionColor);
		}
		else {
			g2.setColor(color);
		}
		g2.draw(arrow);
	}

	public boolean equals(Object object) {
		Edge edge2 = (Edge) object;
		return childVertex.equals(edge2.childVertex)
				&& parentVertex.equals(edge2.parentVertex);
	}

	public Rectangle getBounds() {
		return line.getBounds();
	}

	public Point getCenter() {
		return new Point((int) line.getBounds().getCenterX(), (int) line
				.getBounds().getCenterY());
	}

	public Vertex getChildVertex() {
		return childVertex;
	}

	public Color getColor() {
		return color;
	}

	public Line2D getLine() {
		return line;
	}

	public Vertex getParentVertex() {
		return parentVertex;
	}

	public Color getSelectionColor() {
		return selectionColor;
	}

	public boolean isSelected() {
		return selected;
	}

	public void setCenter(Point center) {
		int tx = (int) (center.getX() - getBounds().getCenterX());
		int ty = (int) (center.getY() - getBounds().getCenterY());
		parentVertex.getCenter().translate(tx, ty);
		childVertex.getCenter().translate(tx, ty);
	}

	public void setColor(Color color) {
		this.color = color;
	}

	public void setSelected(boolean selected) {
		this.selected = selected;
	}

	public void setSelectionColor(Color selectionColor) {
		this.selectionColor = selectionColor;
	}

	public String toString() {
		String result = "";
		if (!name.equals("")) {
			result += name + ": ";
		}
		return result + parentVertex + " -> " + childVertex;
	}

	public void update() {
		line.setLine(parentVertex.getCenter(), childVertex.getCenter());
		if (parentVertex.getBounds().intersects(childVertex.getBounds())) {
			arrow.setLine(new Point2D.Double(0.0, 0.0), new Point2D.Double(0.0,
					0.0));
			return;
		}
		Point2D start = parentVertex.getBounds().findIntersection(line);
		Point2D end = childVertex.getBounds().findIntersection(line);
		if (start == null || end == null) {
			start = new Point2D.Double(0.0, 0.0);
			end = new Point2D.Double(0.0, 0.0);
		}
		arrow.setLine(start, end);
	}
}