/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.graphs;

import java.awt.Color;
import java.awt.Dimension;
import java.util.Vector;

/**
 * A graph consisting of edges and vertices
 * 
 * @author Ralf Ratering
 * @version $Id: Graph.java,v 1.1 2004/05/25 14:58:45 rmenday Exp $
 */
public class Graph {

	/**
	 * colors
	 */
	protected Color color, selectionColor;
	/**
	 * edges
	 */
	protected Vector edges = new Vector();
	/**
	 * vertices
	 */
	protected Vector vertices = new Vector();

	/**
	 * Constructor for the Graph object
	 */
	public Graph() {
		setColor(Color.black);
		setSelectionColor(Color.red);
	}

	public void addEdge(Edge edge) throws Exception {
		if (edges.contains(edge)) {
			throw new Exception("Cannot add edge " + edge
					+ ". Edge already exists.");
		}
		edges.add(edge);
		edge.getParentVertex().addEdge(edge);
		edge.getChildVertex().addEdge(edge);
		edge.setColor(color);
		edge.setSelectionColor(selectionColor);
	}

	/**
	 * Adds a vertex to graph
	 * 
	 * @param vertex
	 *            new vertex in graph
	 */
	public void addVertex(Vertex vertex) {
		if (!vertices.contains(vertex)) {
			vertices.add(vertex);
			vertex.setColor(color);
			vertex.setSelectionColor(selectionColor);
		}
	}

	public Color getColor() {
		return color;
	}

	/**
	 * Gets get the ith edge from list of edges
	 * 
	 * @param i
	 *            edge index in list
	 * @return The Edge value
	 */
	public Edge getEdge(int i) {
		return (Edge) edges.get(i);
	}

	public int getNumberOfEdges() {
		return edges.size();
	}

	public int getNumberOfVertices() {
		return vertices.size();
	}

	public Color getSelectionColor() {
		return selectionColor;
	}

	public Dimension getSize() {
		int width = 0;
		int height = 0;
		for (int i = 0; i < vertices.size(); i++) {
			Vertex vertex = (Vertex) vertices.elementAt(i);
			int xpos = (int) vertex.getBounds().getMaxX();
			int ypos = (int) vertex.getBounds().getMaxY();
			if (xpos > width) {
				width = xpos;
			}
			if (ypos > height) {
				height = ypos;
			}
		}
		return new Dimension(width, height);
	}

	/**
	 * Gets the ith vertex from list of vertices
	 * 
	 * @param i
	 *            vertex index in list
	 * @return The Vertex value
	 */
	public Vertex getVertex(int i) {
		return (Vertex) vertices.get(i);
	}

	public void removeAllEdges() {
		while (edges.size() > 0) {
			removeEdge((Edge) edges.firstElement());
		}
	}

	/**
	 * Remove an edge from the graph
	 * 
	 * @param edge
	 *            edge to be removed
	 */
	public void removeEdge(Edge edge) {
		edges.remove(edge);
		edge.getParentVertex().removeEdge(edge);
		edge.getChildVertex().removeEdge(edge);
	}

	/**
	 * Remove a vertex from the graph
	 * 
	 * @param vertex
	 *            vertex to be removed
	 */
	public void removeVertex(Vertex vertex) {
		vertices.remove(vertex);
		for (int i = 0; i < edges.size(); i++) {
			Edge edge = (Edge) edges.elementAt(i);
			if (edge.contains(vertex)) {
				removeEdge(edge);
			}
		}
	}

	public void setColor(Color color) {
		this.color = color;
		for (int i = 0; i < vertices.size(); i++) {
			((Vertex) vertices.elementAt(i)).setColor(color);
		}
		for (int i = 0; i < edges.size(); i++) {
			((Edge) edges.elementAt(i)).setColor(color);
		}
	}

	public void setSelectionColor(Color selectionColor) {
		this.selectionColor = selectionColor;
		for (int i = 0; i < vertices.size(); i++) {
			((Vertex) vertices.elementAt(i)).setSelectionColor(selectionColor);
		}
		for (int i = 0; i < edges.size(); i++) {
			((Edge) edges.elementAt(i)).setSelectionColor(selectionColor);
		}
	}

	public String toString() {
		String result = "VERTICES:\n";
		for (int i = 0; i < getNumberOfVertices(); i++) {
			result += getVertex(i) + "\n";
		}
		result += "EDGES:\n";
		for (int i = 0; i < getNumberOfEdges(); i++) {
			result += getEdge(i) + "\n";
		}
		return result;
	}
}