/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.graphs;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.util.Vector;
import java.util.logging.Logger;
import javax.swing.ImageIcon;
import javax.swing.JPanel;
import com.pallas.unicore.client.graphics.ClipRectangle;
import com.pallas.unicore.client.graphics.Drawable;
import com.pallas.unicore.client.graphics.Selectable;

/**
 * Representation of a vertex in a graph including an icon, a text a set of
 * edges and a flag if it's selected.
 * 
 * @author Ralf Ratering
 * @version $Id: Vertex.java,v 1.1 2004/05/25 14:58:45 rmenday Exp $
 */
public class Vertex implements Selectable, Drawable {

	protected static Logger logger = Logger
			.getLogger("com.pallas.unicore.client.graphs");
	private Point center = new Point(0, 0);
	private Color color, selectionColor;
	/**
	 * Vector of edges connected to this vertex
	 */
	private Vector edges = new Vector();
	/**
	 * Icon representing vertex
	 */
	private ImageIcon icon;
	/**
	 * Vertex name
	 */
	private String name;
	/**
	 * Flag if vertex is selected
	 */
	private boolean selected;
	private int xpos, ypos, width, height;

	/**
	 * Constructor for the Vertex object
	 */
	public Vertex() {
		this("");
	}

	/**
	 * Constructor for the Vertex object
	 * 
	 * @param name
	 *            vertex name
	 */
	public Vertex(String name) {
		this.name = name;
		setSelected(false);
		setColor(Color.black);
		setSelectionColor(Color.red);
		init();
	}

	public void addEdge(Edge edge) {
		if (!edges.contains(edge)) {
			edges.add(edge);
		}
	}

	/**
	 * Draw icon at its position
	 * 
	 * @param panel
	 *            panel to draw into
	 * @param g
	 *            Description of Parameter
	 */
	public void draw(Graphics g, JPanel panel) {
		int xOffset = 75;
		g.setColor(color);
		// icon
		icon = getImageIcon();
		int iconWidth = icon.getIconWidth();
		int iconHeight = icon.getIconHeight();
		// font
		g.setFont(new Font(null, Font.PLAIN, 10));
		int textHeight = g.getFontMetrics().getHeight();
		int textWidth = g.getFontMetrics().stringWidth(name);
		String displayName = name;
		int end = displayName.length() - 3;
		while (textWidth >= iconWidth + xOffset && end > 0) {
			displayName = name.substring(0, end) + "...";
			textWidth = g.getFontMetrics().stringWidth(displayName);
			end--;
		}
		width = textWidth > iconWidth ? textWidth : iconWidth;
		height = iconHeight + textHeight;
		xpos = (int) (center.getX() - width / 2.0);
		ypos = (int) (center.getY() - height / 2.0);
		g.drawImage(icon.getImage(), (int) (center.getX() - iconWidth / 2.0),
				ypos, panel);
		// draw identifier string centered under icon
		g.drawString(displayName, (int) (center.getX() - textWidth / 2), ypos
				+ height);
		// draw selection box
		if (selected) {
			Graphics2D g2 = (Graphics2D) g;
			g2.setColor(selectionColor);
			g2.fill(getBounds());
		}
	}

	public ClipRectangle getBounds() {
		int border = 4;
		return new ClipRectangle((int) center.getX() - (width + border) / 2,
				(int) center.getY() - (height + border) / 2, width + border,
				height + border);
	}

	public Point getCenter() {
		return center;
	}

	public Color getColor() {
		return color;
	}

	public Vector getEdges() {
		return edges;
	}

	public int getHeight() {
		return height;
	}

	public ImageIcon getImageIcon() {
		return icon;
	}

	public String getName() {
		return name;
	}

	public Color getSelectionColor() {
		return selectionColor;
	}

	public int getWidth() {
		return width;
	}

	public int getXPos() {
		return xpos;
	}

	public int getYPos() {
		return ypos;
	}

	public boolean hasEdgeTo(Vertex neighbor) {
		for (int i = 0; i < edges.size(); i++) {
			if (neighbor.equals(((Edge) edges.elementAt(i)).getChildVertex())) {
				return true;
			}
		}
		return false;
	}

	private void init() {
		// icon
		icon = getImageIcon();
		int iconWidth = icon != null ? icon.getIconWidth() : 60;
		int iconHeight = icon != null ? icon.getIconHeight() : 60;
		// font
		int textHeight = 20;
		int textWidth = 60;
		width = textWidth > iconWidth ? textWidth : iconWidth;
		height = iconHeight + textHeight;
		xpos = (int) (center.getX() - width / 2.0);
		ypos = (int) (center.getY() - height / 2.0);
	}

	public boolean isSelected() {
		return selected;
	}

	public void removeEdge(Edge edge) {
		edges.remove(edge);
	}

	public void setCenter(Point center) {
		this.center = center;
	}

	public void setColor(Color color) {
		this.color = color;
	}

	public void setImageIcon(ImageIcon icon) {
		this.icon = icon;
	}

	public void setName(String name) {
		this.name = name;
	}

	public void setSelected(boolean selected) {
		this.selected = selected;
	}

	public void setSelectionColor(Color sc) {
		// make a transparent copy of color
		this.selectionColor = new Color(sc.getRed(), sc.getGreen(), sc
				.getBlue(), 100);
	}

	/**
	 * Get the String description of this vertex
	 * 
	 * @return String description
	 */
	public String toString() {
		return name;
	}
}