/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.panels;

import java.awt.datatransfer.DataFlavor;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.util.Date;
import java.util.Hashtable;
import java.util.Vector;
import javax.swing.DefaultCellEditor;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableModel;
import org.unicore.Vsite;
import org.unicore.resources.Storage;
import org.unicore.resources.USpace;
import com.pallas.unicore.client.explorer.SelectorDialog;
import com.pallas.unicore.client.remotefilechooser.RemoteFile;
import com.pallas.unicore.client.remotefilechooser.RemoteFileSystem;
import com.pallas.unicore.client.tables.AdjustTable;
import com.pallas.unicore.client.tables.ComboBoxCellRenderer;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.container.TaskContainer;
import com.pallas.unicore.extensions.FileOperation;
import com.pallas.unicore.extensions.FileStorage;
import com.pallas.unicore.extensions.FileTransfer;
import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * @author Thomas Kentemich
 * @author Ralf Ratering
 * @author Klaus-Dieter Oertel
 * @version $Id: FileTransferPanel.java,v 1.1 2004/05/25 14:58:48 rmenday Exp $
 */
public class FileTransferPanel extends FileStoragePanel {

	final static int COL_SOURCE_STORAGE = 0;
	final static int COL_SOURCE = 1;
	final static int COL_TARGET_JOB = 2;
	final static int COL_TARGET_STORAGE = 3;
	final static int COL_TARGET = 4;
	final static int COL_OVERWRITE = 5;
	private JComboBox jobComboBox;
	private JComboBox storagesEditorComboBox;
	private JComboBox targetStoragesEditorComboBox;
	private ComboBoxCellRenderer storagesRendererComboBox;
	private ComboBoxCellRenderer targetStoragesRendererComboBox;
	/**
	 * Vector containing possible destination job groups
	 */
	private Vector jobGroups;
	private Hashtable jobTargetStorages = new Hashtable();

	/**
	 * Constructor
	 * 
	 * @param container
	 *            container to be modified
	 */
	public FileTransferPanel(TaskContainer container) {
		this(container, null);
	}

	/**
	 * Constructor for the FileTransferPanel object
	 * 
	 * @param container
	 *            Description of the Parameter
	 * @param initialDirectory
	 *            Description of the Parameter
	 */
	public FileTransferPanel(TaskContainer container, String initialDirectory) {
		super(container, initialDirectory);
		buildComponents();
	}

	/**
	 * Add and initialize a new empty row to the table
	 */
	protected void addEmptyRow() {
		Vector lastRowData = null;
		int selectedRow = table.getSelectedRow();
		if (selectedRow >= 0) {
			lastRowData = (Vector) tableModel.getDataVector().elementAt(
					selectedRow);
		}
		Vector rowData = new Vector();
		// set defaults if there is no row in the table
		if (lastRowData == null) {
			// determine Uspace at source as the default setting
			USpace uspaceSource;
			if (container.getVsite() != null) {
				uspaceSource = ResourceManager.getResourceSet(
						container.getVsite()).getUspace();
			}
			else {
				uspaceSource = new USpace();
			}
			// determine target job and Uspace at target as default setting
			Object jc = null;
			USpace uspaceTarget = null;
			if (jobGroups != null) {
				if (jobGroups.size() == 1) {
					jc = (JobContainer) jobGroups.elementAt(0);
					Vsite vsiteTarget = ((JobContainer) jc).getVsite();
					if (vsiteTarget != null) {
						uspaceTarget = ResourceManager.getResourceSet(
								vsiteTarget).getUspace();
					}
				}
			}
			if (uspaceTarget == null) {
				uspaceTarget = new USpace();
			}
			rowData.add(uspaceSource);
			rowData.add("");
			rowData.add(jc);
			rowData.add(uspaceTarget);
			rowData.add("");
			rowData.add(Boolean.FALSE);
		}
		else {
			rowData.add(lastRowData.elementAt(COL_SOURCE_STORAGE));
			rowData.add("");
			rowData.add(lastRowData.elementAt(COL_TARGET_JOB));
			rowData.add(lastRowData.elementAt(COL_TARGET_STORAGE));
			rowData.add("");
			rowData.add(lastRowData.elementAt(COL_OVERWRITE));
		}
		tableModel.addRow(rowData);
		selectedRow = table.getRowCount() - 1;
		table.setRowSelectionInterval(selectedRow, selectedRow);
	}

	/**
	 * Apply gui values to container
	 */
	public void applyValues() {
		// stop editing
		if (table.getCellEditor() != null) {
			table.getCellEditor().stopCellEditing();
		}
		// get file transfers from table
		int nrOfRows = table.getRowCount();
		FileTransfer[] fileOps = new FileTransfer[nrOfRows];
		for (int i = 0; i < nrOfRows; i++) {
			fileOps[i] = (FileTransfer) getFileOperationAt(i);
		}
		updateModifiedTime(fileOps);
		container.setFileTransfers(fileOps);
		container.checkContents();
	}

	private void buildComponents() {
		// minimize checkbox and combobox columns
		int storageChoice = 70;
		table.getColumnModel().getColumn(COL_SOURCE_STORAGE).setPreferredWidth(
				storageChoice);
		// table.getColumnModel().getColumn(COL_SOURCE_STORAGE).setMinWidth(storageChoice);
		// table.getColumnModel().getColumn(COL_SOURCE_STORAGE).setMaxWidth(storageChoice);
		storageChoice = 10;
		table.getColumnModel().getColumn(COL_TARGET_STORAGE).setPreferredWidth(
				storageChoice);
		// table.getColumnModel().getColumn(COL_TARGET_STORAGE).setMinWidth(storageChoice);
		// table.getColumnModel().getColumn(COL_TARGET_STORAGE).setMaxWidth(storageChoice);
		table.getColumnModel().getColumn(COL_TARGET).setPreferredWidth(
				storageChoice);
		// table.getColumnModel().getColumn(COL_TARGET).setMinWidth(storageChoice);
		// table.getColumnModel().getColumn(COL_TARGET).setMaxWidth(storageChoice);
		// build the combobox editors and renderers in tables
		storages = new Vector();
		storagesEditorComboBox = new JComboBox(storages);
		storagesRendererComboBox = new ComboBoxCellRenderer(storages);
		table.getColumnModel().getColumn(COL_SOURCE_STORAGE).setCellEditor(
				new DefaultCellEditor(storagesEditorComboBox));
		table.getColumnModel().getColumn(COL_SOURCE_STORAGE).setCellRenderer(
				storagesRendererComboBox);
		// build the combobox editors and renderers in tables
		targetStorages = new Vector();
		targetStoragesEditorComboBox = new JComboBox(targetStorages);
		targetStoragesRendererComboBox = new ComboBoxCellRenderer(
				targetStorages);
		table.getColumnModel().getColumn(COL_TARGET_STORAGE).setCellEditor(
				new DefaultCellEditor(targetStoragesEditorComboBox));
		table.getColumnModel().getColumn(COL_TARGET_STORAGE).setCellRenderer(
				targetStoragesRendererComboBox);
		jobGroups = new Vector();
		jobComboBox = new JComboBox(jobGroups);
		jobComboBox.setSelectedItem(null);
		table.getColumnModel().getColumn(COL_TARGET_JOB).setCellEditor(
				new DefaultCellEditor(jobComboBox));
		table.getColumnModel().getColumn(COL_TARGET_JOB).setCellRenderer(
				new ComboBoxCellRenderer(jobGroups));
		ComboBoxListener comboBoxListener = new ComboBoxListener();
		jobComboBox.addItemListener(comboBoxListener);
		table.adjustWidths();
	}

	protected AdjustTable createTable() {
		return new FileTransferTable(tableModel);
	}

	protected String[] getColumnNames() {
		String[] columnNames = {res.getString("SOURCE"),
				res.getString("FILE_AT_SOURCE"),
				res.getString("DESTINATION_JOB"), res.getString("DESTINATION"),
				res.getString("FILE_AT_DESTINATION"),
				res.getString("OVERWRITE")};
		return columnNames;
	}

	protected DataFlavor getDataFlavor() {
		return FileTransfer.getFlavor();
	}

	/**
	 * Get file transfer from a certain row
	 * 
	 * @param row
	 *            index of row
	 * @return file transfer from row
	 */
	protected FileOperation getFileOperationAt(int row) {
		Object sourceStorage = tableModel.getValueAt(row, COL_SOURCE_STORAGE);
		String sourceName = (String) tableModel.getValueAt(row, COL_SOURCE);
		Object destinationJobEntry = tableModel.getValueAt(row, COL_TARGET_JOB);
		JobContainer destinationJob = null;
		if (destinationJobEntry instanceof JobContainer) {
			destinationJob = (JobContainer) destinationJobEntry;
		}
		Storage destinationStorage = (Storage) tableModel.getValueAt(row,
				COL_TARGET_STORAGE);
		String destinationName = (String) tableModel
				.getValueAt(row, COL_TARGET);
		boolean overwrite = ((Boolean) tableModel
				.getValueAt(row, COL_OVERWRITE)).booleanValue();
		// renaming not yet implemented...
		FileTransfer fileTransfer = new FileTransfer(sourceStorage, sourceName,
				destinationJob, destinationStorage, destinationName);
		fileTransfer.setOverwritePolicy(overwrite);
		return fileTransfer;
	}

	/**
	 * Reset current GUI entries to last container contents.
	 */
	public void resetValues() {
		// check if container has id and name
		if (container.getIdentifier() == null) {
			return;
		}
		// stop editing
		if (table.getCellEditor() != null) {
			table.getCellEditor().stopCellEditing();
		}
		clearTable();
		updateDestinationJobGroups();
		FileTransfer[] fileOps = container.getFileTransfers();
		for (int i = 0; i < fileOps.length; i++) {
			addFileOperation(fileOps[i]);
		}
	}

	/**
	 * Fill an existing table row with a certain FileTransfer object. If row
	 * index is too high add a new row and use that one.
	 * 
	 * @param index
	 *            nr of table row
	 * @param fileOp
	 *            The new fileOperationAt value
	 */
	protected void setFileOperationAt(FileOperation fileOp, int index) {
		FileTransfer fileTransfer = (FileTransfer) fileOp;
		String target = fileTransfer.getDestinationName();
		Boolean overwrite = new Boolean(fileTransfer.getOverwritePolicy());
		// the table is edited
		if (index < table.getRowCount()) {
			String oldSource = (String) tableModel
					.getValueAt(index, COL_SOURCE);
			String oldTarget = (String) tableModel
					.getValueAt(index, COL_TARGET);
			// if table entry already contains a target name, keep it
			if (!oldTarget.equals("")) {
				target = oldTarget;
			}
			// if table entry already contains a source or target name, keep the
			// flags
			if (!oldSource.equals("") || !oldTarget.equals("")) {
				overwrite = (Boolean) tableModel.getValueAt(index,
						COL_OVERWRITE);
			}
		}
		if (index > table.getRowCount() - 1) {
			tableModel.addRow(new Vector());
		}
		// set source storage
		// (correct null values of previous Client versions)
		if (fileTransfer.getStorageItem() == null) {
			USpace uspaceSource = null;
			if (container.getVsite() != null) {
				uspaceSource = ResourceManager.getResourceSet(
						container.getVsite()).getUspace();
			}
			if (uspaceSource == null) {
				uspaceSource = new USpace();
			}
			tableModel.setValueAt(uspaceSource, index, COL_SOURCE_STORAGE);
		}
		else {
			tableModel.setValueAt(fileTransfer.getStorageItem(), index,
					COL_SOURCE_STORAGE);
		}
		// set source name
		tableModel.setValueAt(fileTransfer.getSourceName(), index, COL_SOURCE);
		// set destination job
		JobContainer jc = fileTransfer.getDestinationJob();
		tableModel.setValueAt(jc, index, COL_TARGET_JOB);
		// set destination storage
		// (correct null values of previous Client versions)
		if (fileTransfer.getOtherStorage() == null) {
			USpace uspaceTarget = null;
			if (jc != null && jc.getVsite() != null) {
				uspaceTarget = ResourceManager.getResourceSet(jc.getVsite())
						.getUspace();
			}
			if (uspaceTarget == null) {
				uspaceTarget = new USpace();
			}
			//			fileTransfer.setOtherStorage(uspaceTarget);
			tableModel.setValueAt(uspaceTarget, index, COL_TARGET_STORAGE);
		}
		else {
			tableModel.setValueAt(fileTransfer.getOtherStorage(), index,
					COL_TARGET_STORAGE);
		}
		// set destination name
		tableModel.setValueAt(target, index, COL_TARGET);
		// set overwrite flag
		tableModel.setValueAt(overwrite, index, COL_OVERWRITE);
	}

	/**
	 * Popup the remote file chooser
	 */
	protected void showBrowser() {
		File selectedFile = null;
		if (table.getSelectedRow() >= 0) {
			FileTransfer fileTransfer = (FileTransfer) getFileOperationAt(table
					.getSelectedRow());
			logger.fine("Selected file transfer: " + fileTransfer);
			if (fileTransfer != null) {
				Object storageItem = fileTransfer.getStorageItem();
				String filename = fileTransfer.getSourceName();
				selectedFile = new RemoteFile(filename, (Storage) storageItem,
						new RemoteFileSystem(container.getVsite()));
			}
		}
		logger.fine("Selected file: " + selectedFile);
		SelectorDialog chooser = ResourceManager.getSelectorDialog(
				selectedFile, container.getVsite());
		chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		chooser.setTitle("UNICORE: Choose source for import");
		int result = chooser.showDialog();
		if (result == JFileChooser.APPROVE_OPTION) {
			// for now only single selection is supported
			File[] files = {chooser.getSelectedFile()};
			for (int i = 0; i < files.length; i++) {
				selectedFile = files[i];
				Object storageObject = null;
				String path = selectedFile.getPath();
				String name = selectedFile.getName();
				if (selectedFile instanceof RemoteFile) {
					RemoteFile file = (RemoteFile) selectedFile;
					storageObject = file.getStorage();
					path = file.getSubspacePath();
					// always add a slash to path for remote directories
					if (selectedFile.isDirectory()) {
						path += "/";
						name += "/";
					}
				}
				FileTransfer fileTransfer = new FileTransfer(storageObject,
						path, null, name);
				// if we have selected multiple files we want to add them
				// if we selected one table row and one file we wanted to edit
				if (table.getSelectedRowCount() == 1 && files.length == 1) {
					setFileOperationAt(fileTransfer, table.getSelectedRow());
				}
				else {
					addFileOperation(fileTransfer);
				}
			}
		}
	}

	/**
	 * Add all possible destination sub job groups to combo box
	 */
	private void updateDestinationJobGroups() {
		JobContainer parentJob = container.getParentJobContainer();
		JobContainer topParentJob = parentJob;
		while (topParentJob.getParentContainer() != null) {
			topParentJob = topParentJob.getParentJobContainer();
		}
		jobGroups = topParentJob.getSubJobs();
		// add top level job itself
		jobGroups.add(topParentJob);
		// remove own job from vector
		for (int i = 0; i < jobGroups.size(); i++) {
			JobContainer jc = (JobContainer) jobGroups.elementAt(i);
			if (jc.getIdentifier().equals(parentJob.getIdentifier())) {
				jobGroups.remove(jc);
			}
		}
		// build hash table of Vsites and associated storages
		jobTargetStorages.clear();
		for (int j = 0; j < jobGroups.size(); j++) {
			Vsite vsite = ((JobContainer) jobGroups.elementAt(j)).getVsite();
			if (vsite != null) {
				if (!jobTargetStorages.containsKey(ResourceManager
						.getUniqueVsiteIdentifier(vsite))) {
					Vector targetStorages = updateStorageResources(vsite);
					for (int i = 0; i < targetStorages.size(); i++) {
						if (targetStorages.elementAt(i).equals(
								FileStorage.NSPACE_STRING)) {
							targetStorages.remove(targetStorages.elementAt(i));
							break;
						}
					}
					jobTargetStorages.put(ResourceManager
							.getUniqueVsiteIdentifier(vsite), targetStorages);
				}
			}
		}
		// JComboBox jobComboBox = new JComboBox(jobGroups);
		jobComboBox.setModel(new DefaultComboBoxModel(jobGroups));
		jobComboBox.setSelectedItem(null);
		table.getColumnModel().getColumn(COL_TARGET_JOB).setCellEditor(
				new DefaultCellEditor(jobComboBox));
		table.getColumnModel().getColumn(COL_TARGET_JOB).setCellRenderer(
				new ComboBoxCellRenderer(jobGroups));
	}

	private void updateModifiedTime(FileTransfer[] newOps) {
		FileTransfer[] oldOps = container.getFileTransfers();
		// first a simple check...
		if (oldOps.length != newOps.length) {
			container.setModifiedTime(new Date());
			return;
		}
		// ... now try to find matching ops
		// loop over old ops
		for (int i = 0; i < oldOps.length; i++) {
			boolean found = false;
			// loop over new ops
			for (int j = 0; j < newOps.length; j++) {
				// try to find matching new import
				if (newOps[j].equals(oldOps[i])) {
					// match was found
					found = true;
				}
			}
			if (!found) {
				// if we get here one op was new...
				container.setModifiedTime(new Date());
				return;
			}
		}
	}

	/**
	 * Update storage resource entries
	 */
	public void updateValues() {
		//  Update destination job groups
		updateDestinationJobGroups();
		// replace existing storage resources
		replaceJobGroups();
		// replace existing storage resources
		replaceTargetStorageResources(COL_TARGET_STORAGE);
		// get storage resources from currently selected vsite
		updateStorageResources();
		// replace existing storage resources
		replaceStorageResources(COL_SOURCE_STORAGE);
		for (int i = 0; i < storages.size(); i++) {
			if (storages.elementAt(i).equals(FileStorage.NSPACE_STRING)) {
				storages.remove(storages.elementAt(i));
				break;
			}
		}
		// update combo boxes
		storagesEditorComboBox.setModel(new DefaultComboBoxModel(storages));
		storagesRendererComboBox.setModel(new DefaultComboBoxModel(storages));
	}

	protected void replaceStorageResources(int col) {
		if (col != COL_SOURCE_STORAGE) {
			super.replaceStorageResources(col);
			return;
		}
		for (int i = 0; i < table.getRowCount(); i++) {
			Object storage = tableModel.getValueAt(i, col);
			// do not replace "Local"
			if (storage instanceof Storage) {
				//ResourceEntry siteStorage =
				// storages.findResourceEntry(((ResourceEntry)storage).getResource());
				NamedResourceSet siteResources = ResourceManager
						.getResourceSet(container.getVsite());
				Storage siteStorage = (Storage) siteResources
						.findResource((Storage) storage);
				// Only replace storages found at current Vsite; keep "wrong"
				// storage from other Vsite
				if (siteStorage != null) {
					tableModel.setValueAt(siteStorage, i, col);
				}
			}
			else if (storage == null) {
				if (container.getVsite() != null) {
					USpace uspace = ResourceManager.getResourceSet(
							container.getVsite()).getUspace();
					tableModel.setValueAt(uspace, i, col);
				}
				else {
					USpace uspace = new USpace();
					tableModel.setValueAt(uspace, i, col);
				}
			}
		}
		// apply new entries
		applyValues();
	}

	protected void replaceJobGroups() {
		for (int i = 0; i < table.getRowCount(); i++) {
			JobContainer jc = (JobContainer) tableModel.getValueAt(i,
					COL_TARGET_JOB);
			// replace empty job container if only one job container available
			if (jc == null && jobGroups.size() == 1) {
				jc = (JobContainer) jobGroups.elementAt(0);
				tableModel.setValueAt(jc, i, COL_TARGET_JOB);
			}
		}
	}

	protected void replaceTargetStorageResources(int col) {
		for (int i = 0; i < table.getRowCount(); i++) {
			JobContainer jc = (JobContainer) tableModel.getValueAt(i,
					COL_TARGET_JOB);
			// replace storages
			if (jc != null && jc.getVsite() != null) {
				Object storage = tableModel.getValueAt(i, col);
				// do not replace "Local"
				if (storage instanceof Storage) {
					//ResourceEntry siteStorage =
					// storages.findResourceEntry(((ResourceEntry)storage).getResource());
					NamedResourceSet siteResources = ResourceManager
							.getResourceSet(jc.getVsite());
					Storage siteStorage = (Storage) siteResources
							.findResource((Storage) storage);
					// Only replace storages found at current Vsite; keep
					// "wrong" storage from other Vsite
					if (siteStorage != null) {
						tableModel.setValueAt(siteStorage, i, col);
					}
				}
				else if (storage == null) {
					USpace uspace = ResourceManager.getResourceSet(
							jc.getVsite()).getUspace();
					if (uspace == null) {
						uspace = new USpace();
					}
					tableModel.setValueAt(uspace, i, col);
				}
			}
		}
		// apply new entries
		applyValues();
	}

	private class ComboBoxListener implements ItemListener {

		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.DESELECTED) {
				//int rows[] = table.getSelectedRows();
				//for (int i=0; i<rows.length; i++) {
				// System.err.println("DESELECTED "+e.getItem()+" rows["+i+" =
				// "+rows[i]);}
				return;
			}
			if (e.getSource() == jobComboBox) {
				//int rows[] = table.getSelectedRows();
				//for (int i=0; i<rows.length; i++) { System.err.println("
				// SELECTED "+e.getItem()+" rows["+i+" = "+rows[i]);}
				int row = table.getSelectedRow();
				Object obj = e.getItem();
				JobContainer jc = (JobContainer) obj;
				Vsite vsite = jc.getVsite();
				if (vsite != null) {
					targetStorages = (Vector) jobTargetStorages
							.get(ResourceManager
									.getUniqueVsiteIdentifier(vsite));
				}
				else {
					targetStorages = new Vector();
				}
				targetStoragesEditorComboBox.setModel(new DefaultComboBoxModel(
						targetStorages));
				targetStoragesRendererComboBox
						.setModel(new DefaultComboBoxModel(targetStorages));
				tableModel.fireTableRowsUpdated(row, row);
			}
		}
	}

	/**
	 * Table with cell dependent renderers/editors
	 */
	class FileTransferTable extends AdjustTable {

		public FileTransferTable(TableModel tableModel) {
			super(tableModel);
		}

		public TableCellEditor getCellEditor(int row, int col) {
			if (convertColumnIndexToModel(col) == COL_TARGET_STORAGE) {
				Vector targetStorages;
				JobContainer jc = (JobContainer) tableModel.getValueAt(row,
						COL_TARGET_JOB);
				if (jc != null && jc.getVsite() != null) {
					targetStorages = (Vector) jobTargetStorages
							.get(ResourceManager.getUniqueVsiteIdentifier(jc
									.getVsite()));
					// targetStorages might be Null if transfer task was pasted
					// into the target job
					if (targetStorages == null) {
						targetStorages = new Vector();
					}
				}
				else {
					targetStorages = new Vector();
				}
				DefaultCellEditor targetStorageCellEditor = (DefaultCellEditor) super
						.getCellEditor(row, col);
				JComboBox jcb = (JComboBox) targetStorageCellEditor
						.getComponent();
				jcb.setModel(new DefaultComboBoxModel(targetStorages));
				return targetStorageCellEditor;
			}
			else {
				return super.getCellEditor(row, col);
			}
		}

		public TableCellRenderer getCellRenderer(int row, int col) {
			if (convertColumnIndexToModel(col) == COL_TARGET_STORAGE) {
				Vector targetStorages;
				JobContainer jc = (JobContainer) tableModel.getValueAt(row,
						COL_TARGET_JOB);
				if (jc != null && jc.getVsite() != null) {
					targetStorages = (Vector) jobTargetStorages
							.get(ResourceManager.getUniqueVsiteIdentifier(jc
									.getVsite()));
					// targetStorages might be Null if transfer task was pasted
					// into the target job
					if (targetStorages == null) {
						targetStorages = new Vector();
					}
				}
				else {
					targetStorages = new Vector();
				}
				targetStoragesRendererComboBox
						.setModel(new DefaultComboBoxModel(targetStorages));
				return targetStoragesRendererComboBox;
			}
			else {
				return super.getCellRenderer(row, col);
			}
		}
	}
}