/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.panels;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Vector;
import java.util.logging.Logger;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.UIManager;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import org.unicore.outcome.AbstractActionStatus;
import com.pallas.unicore.client.tables.AdjustTable;
import com.pallas.unicore.client.tables.DateTableCellRenderer;
import com.pallas.unicore.resourcemanager.DetailsEntry;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.VectorSorter;

/**
 * @author Ralf Ratering
 * @version $Id: OutcomeDetailsTable.java,v 1.1 2004/05/25 14:58:48 rmenday Exp $
 */
public class OutcomeDetailsTable extends AdjustTable {

	protected class DetailsTableModel extends DefaultTableModel {

		public boolean isCellEditable(int row, int col) {
			return false;
		}
	}

	protected static class StatusTableCellRenderer implements TableCellRenderer {

		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column) {
			JLabel label = new JLabel(value.toString());
			label.setFont(table.getFont());
			if (isSelected) {
				label.setBackground(table.getSelectionBackground());
			}
			else {
				label.setBackground(Color.WHITE);
			}
			if (value instanceof AbstractActionStatus) {
				AbstractActionStatus status = (AbstractActionStatus) value;
				if (status.isEquivalent(AbstractActionStatus.NOT_SUCCESSFUL)) {
					label.setForeground(Color.RED);
				}
				else if (status.isEquivalent(AbstractActionStatus.SUCCESSFUL)) {
					label.setForeground(Color.GREEN);
				}
			}
			label.setOpaque(true);
			return label;
		}
	}

	protected class TableHeaderListener extends MouseAdapter {

		private OutcomeDetailsTable table;

		public TableHeaderListener() {
		}

		public void mouseClicked(MouseEvent e) {
			TableColumnModel columnModel = getColumnModel();
			int viewColumn = columnModel.getColumnIndexAtX(e.getX());
			sortColumn = convertColumnIndexToModel(viewColumn);
			sortByColumn(sortColumn);
		}
	}

	private class TableHeaderRenderer extends DefaultTableCellRenderer {

		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column) {
			if (column == sortColumn) {
				if (ascending) {
					this.setIcon(up);
				}
				else {
					this.setIcon(down);
				}
			}
			else {
				this.setIcon(null);
			}
			this.setHorizontalTextPosition(JLabel.LEFT);
			JTableHeader header = table.getTableHeader();
			if (header != null) {
				this.setForeground(header.getForeground());
				this.setBackground(header.getBackground());
				this.setFont(header.getFont());
			}
			this.setText((value == null) ? "" : value.toString());
			this.setBorder(UIManager.getBorder("TableHeader.cellBorder"));
			return this;
		}
	}
	private final static String[] columnIdentifiers = {"Time Stamp", "Name",
			"Description", "Status", "Reason", "Iteration"};
	private static TableCellRenderer dateCellRenderer = new DateTableCellRenderer(
			ResourceManager.getCompleteFormatter());
	private static Logger logger = Logger
			.getLogger("com.pallas.unicore.client.panels");
	private static TableCellRenderer statusCellRenderer = new StatusTableCellRenderer();
	private boolean ascending;
	private final ImageIcon down = ResourceManager
			.getIcon(ResourceManager.DOWN);
	private boolean initialized = false;
	private int lastSortColumn = -1;
	private DetailsTableModel model;
	private int sortColumn = 0;
	private final ImageIcon up = ResourceManager.getIcon(ResourceManager.UP);

	public OutcomeDetailsTable() {
		setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		buildComponents();
		getTableHeader().setDefaultRenderer(new TableHeaderRenderer());
		getTableHeader().addMouseListener(new TableHeaderListener());
	}

	private void adjustWidthsAndLookForIterations(Vector rows) {
		adjustWidths();
		boolean hasIteration = false;
		for (int i = 0; i < rows.size(); i++) {
			DetailsEntry entry = (DetailsEntry) rows.elementAt(i);
			if (!entry.getIteration().equals("")) {
				hasIteration = true;
				break;
			}
		}
		if (!hasIteration) {
			TableColumn column = getColumn("Iteration");
			column.setMinWidth(0);
			column.setMaxWidth(0);
			column.setPreferredWidth(0);
			column.setResizable(false);
		}
	}

	private void buildComponents() {
		model = new DetailsTableModel();
		model.setColumnIdentifiers(columnIdentifiers);
		setModel(model);
		getColumnModel().getColumn(0).setCellRenderer(dateCellRenderer);
		getColumnModel().getColumn(3).setCellRenderer(statusCellRenderer);
	}

	public void clear() {
		int rowCount = getRowCount();
		for (int i = 0; i < rowCount; i++) {
			model.removeRow(0);
		}
	}

	public DetailsEntry getSelectedEntry() {
		return (DetailsEntry) model.getDataVector().elementAt(getSelectedRow());
	}

	public void resort() {
		ascending = !ascending;
		sortByColumn(lastSortColumn);
	}

	public void setDataVector(Vector rows) {
		this.clear();
		for (int i = 0; i < rows.size(); i++) {
			model.addRow((Vector) rows.elementAt(i));
		}
		resort();
		if (rows.size() > 0 && !initialized) {
			adjustWidthsAndLookForIterations(rows);
			initialized = true;
		}
	}

	public void sortByColumn(int column) {
		if (column < 0) {
			return;
		}
		if (lastSortColumn == column) {
			ascending = !ascending;
		}
		else {
			ascending = true;
		}
		DetailsEntry.setCompareIndex(column, 0);
		VectorSorter.sort(model.getDataVector(), ascending);
		lastSortColumn = column;
	}
}