/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.plugins.script;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Enumeration;
import java.util.ResourceBundle;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;
import org.unicore.ajo.ScriptType;
import com.pallas.unicore.client.controls.FontSelectionComboBox;
import com.pallas.unicore.client.controls.LayoutTools;
import com.pallas.unicore.client.dialogs.GenericDialog;
import com.pallas.unicore.client.explorer.SelectorDialog;
import com.pallas.unicore.client.panels.FontSelectionPanel;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.UserMessages;

/**
 * User interface to change script default settings
 * 
 * @author Ralf Ratering
 * @version $Id: ScriptDefaultsDialog.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public class ScriptDefaultsDialog extends GenericDialog {

	/**
	 * Listen to button clicks
	 */
	private class ButtonListener implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			Object source = e.getSource();
			if (source == scriptDirButton) {
				String value = chooseDirectory(res.getString("DIR_TITLE"),
						scriptDefaults.getScriptDirectory());
				if (value != null) {
					scriptDirTextField.setText(value);
				}
			}
		}
	}
	static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.client.plugins.script.ResourceStrings");
	private FontSelectionPanel fontPanel;
	private JCheckBox lineWrapCheckBox = new JCheckBox("Line wrap");
	private ScriptDefaults scriptDefaults;
	private JButton scriptDirButton;
	private JTextField scriptDirTextField;
	private JComboBox typeComboBox;

	/**
	 * Constructor takes parent frame as input and builds modal dialog
	 * 
	 * @param parentFrame
	 *            reference to parent JFrame
	 * @param scriptDefaults
	 */
	public ScriptDefaultsDialog(JFrame parentFrame,
			ScriptDefaults scriptDefaults) {
		super(parentFrame, res.getString("DEFAULTS_TITLE"), true, OK_CANCEL);
		this.scriptDefaults = scriptDefaults;
		buildComponents();
		resetValues();
		setStatusMessage(res.getString("ENTER_VALUES"));
		pack();
	}

	/**
	 * Add tooltips to buttons from super class
	 */
	private void addTooltipsToButtons() {
		okButton.setToolTipText(res.getString("STORE_AND_CLOSE"));
		applyButton.setToolTipText(res.getString("STORE_TIP"));
		cancelButton.setToolTipText(res.getString("CLOSE_TIP"));
	}

	/**
	 * Method will be called from parent class
	 * 
	 * @return true if applying was successful
	 */
	protected boolean applyValues() {
		// copy text field entries to user defaults
		scriptDefaults.setScriptDirectory(scriptDirTextField.getText().trim());
		scriptDefaults.setTypeString((String) typeComboBox.getSelectedItem());
		scriptDefaults.setEditorFont(fontPanel.getSelectedFont());
		scriptDefaults.setLineWrap(lineWrapCheckBox.isSelected());
		// write defaults to file
		if (!scriptDefaults.writeToFile()) {
			UserMessages.error(res.getString("COULD_NOT_WRITE"));
			setStatusMessage(res.getString("COULD_NOT_WRITE"));
		}
		else {
			setStatusMessage(res.getString("SUCCESS"));
		}
		// returning true closes dialog if ok was pressed
		return true;
	}

	/**
	 * Build user panel
	 */
	private void buildComponents() {
		JPanel mainPanel = new JPanel();
		GridBagLayout g = new GridBagLayout();
		mainPanel.setLayout(g);
		GridBagConstraints c = new GridBagConstraints();
		c = LayoutTools.makegbc(0, 0, 1, 0, false);
		mainPanel.add(new JLabel(res.getString("SCRIPT_DIR")), c);
		// fill text fields with previous defaults
		scriptDirTextField = new JTextField(25);
		c = LayoutTools.makegbc(1, 0, 2, 100, true);
		mainPanel.add(scriptDirTextField, c);
		// file chooser buttons
		scriptDirButton = new JButton(ResourceManager
				.getIcon(ResourceManager.OPEN));
		scriptDirButton.addActionListener(new ButtonListener());
		c = LayoutTools.makegbc(3, 0, 1, 0, false);
		mainPanel.add(scriptDirButton);
		c = LayoutTools.makegbc(0, 1, 1, 0, false);
		mainPanel.add(new JLabel(res.getString("TYPE_LABEL")), c);
		// script type
		typeComboBox = new JComboBox();
		Enumeration typeEnum = ScriptType.elements();
		while (typeEnum.hasMoreElements()) {
			ScriptType type = (ScriptType) typeEnum.nextElement();
			typeComboBox.addItem(type.toString());
		}
		typeComboBox.setToolTipText(res.getString("SCRIPT_TYPE"));
		c = LayoutTools.makegbc(1, 1, 1, 100, true);
		mainPanel.add(typeComboBox, c);
		c = LayoutTools.makegbc(2, 1, 1, 0, false);
		c.anchor = GridBagConstraints.EAST;
		mainPanel.add(lineWrapCheckBox, c);
		c = LayoutTools.makegbc(0, 2, 1, 0, false);
		mainPanel.add(new JLabel(res.getString("FONT_LABEL")), c);
		// script editor font
		fontPanel = new FontSelectionPanel();
		FontSelectionComboBox fontBox = fontPanel.getFontComboBox();
		fontBox.setToolTipText(res.getString("SCRIPT_FONT"));
		c = LayoutTools.makegbc(1, 2, 1, 100, true);
		mainPanel.add(fontBox, c);
		c = LayoutTools.makegbc(2, 2, 1, 0, false);
		c.anchor = GridBagConstraints.EAST;
		mainPanel.add(fontPanel.getSizeLabel(), c);
		JComboBox sizeBox = fontPanel.getSizeComboBox();
		sizeBox.setToolTipText(res.getString("SCRIPT_FONT"));
		c = LayoutTools.makegbc(3, 2, 1, 0, false);
		c.anchor = GridBagConstraints.EAST;
		mainPanel.add(sizeBox, c);
		mainPanel.setBorder(new EmptyBorder(10, 10, 10, 10));
		getContentPane().add(mainPanel, BorderLayout.CENTER);
		addTooltipsToButtons();
	}

	/**
	 * When cancel was pressed reload user defaults from file
	 * 
	 * @return if canceling was successful
	 */
	protected boolean cancelValues() {
		scriptDefaults.loadFromFile();
		resetValues();
		return true;
	}

	/**
	 * Popup file chooser and return the selected directory
	 * 
	 * @param title
	 *            dialog title
	 * @param initialdir
	 *            start directory
	 * @return selected directory as String
	 */
	private String chooseDirectory(String title, String initialdir) {
		SelectorDialog chooser = ResourceManager.getSelectorDialog(new File(
				initialdir), null);
		chooser.setTitle(title);
		chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		int result = chooser.showDialog();
		if (result == JFileChooser.APPROVE_OPTION) {
			return chooser.getSelectedFile().getAbsolutePath();
		}
		return null;
	}

	/**
	 * Fill controls with current values
	 */
	protected void resetValues() {
		scriptDirTextField.setText(scriptDefaults.getScriptDirectory());
		typeComboBox.setSelectedItem(scriptDefaults.getTypeString());
		fontPanel.setSelectedFont(scriptDefaults.getEditorFont());
		lineWrapCheckBox.setSelected(scriptDefaults.getLineWrap());
	}

	/**
	 * Overwrite show method to let email text field always grab focus.
	 */
	public void show() {
		scriptDirTextField.requestFocus();
		getRootPane().setDefaultButton(okButton);
		super.show();
	}
}