/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.remotefilechooser;

import java.io.File;
import java.io.FileFilter;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.ResourceBundle;
import java.util.logging.Logger;
import org.unicore.Vsite;
import org.unicore.outcome.XDirectory;
import org.unicore.outcome.XFile;
import org.unicore.resources.Storage;

/**
 * Subclass of File that builds a wrapper layer around AJO XFiles to represent
 * them inside a Java file chooser. NOTE: We cannot build an arbitrary new
 * RemoteFile from scratch. We can only create new Roots, and then incrementally
 * build RemoteFiles down the directory hierarchy given the parent file and the
 * new XFile to be wrapped.
 * 
 * @author Ralf Ratering
 * @version $Id: RemoteFile.java,v 1.1 2004/05/25 14:58:55 rmenday Exp $
 */
public class RemoteFile extends File {

	private static Logger logger = Logger
			.getLogger("com.pallas.unicore.client.remotefilechooser");
	// storage location of xfile
	static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.client.remotefilechooser.ResourceStrings");
	/**
	 * The file separator at the remote file system
	 */
	public final static String separatorChar = RemoteFileSystem.SEPARATOR;
	final static long serialVersionUID = -8646290899749020315L;

	/**
	 * Do nothing yet
	 * 
	 * @return null
	 */
	public static File[] listRoots() {
		return (File[]) null;
	}
	private RemoteFileSystem fs;
	// parent RemoteFile in directory tree
	private RemoteFile parent;
	// path as shown in file chooser
	// This is not the "real" path in the file system, but instead consists
	// of a subspace part and a subspace path, e.g:
	// "Home Home/tmp/test.txt"
	private String path;
	private Storage storage;
	private String subspacePath;
	// wrapped xfile
	private XFile xfile;

	/**
	 * Constructor for a root file representing a storage subspace
	 * 
	 * @param storage
	 *            the storage subspace
	 * @param fs
	 */
	public RemoteFile(Storage storage, RemoteFileSystem fs) {
		super(storage.toString());
		this.path = storage.toString();
		this.storage = storage;
		this.fs = fs;
		// build a new XDirectory for this root
		xfile = new XDirectory();
		xfile.setCanExecute(true);
		xfile.setCanRead(true);
	}

	public RemoteFile(String subspacePath, Storage storage, RemoteFileSystem fs) {
		super(storage.toString() + separatorChar + subspacePath);
		this.subspacePath = subspacePath;
		this.storage = storage;
		this.fs = fs;
		this.xfile = new XFile();
		xfile.setName(fs.getPath(storage) + separatorChar + subspacePath);
		path = storage.toString() + separatorChar + subspacePath;
		// build parent as new RemoteFile from subspacePath
		int index = subspacePath.indexOf(separator);
		if (index > 0) {
			String parentPath = subspacePath.substring(0, index);
			parent = new RemoteFile(parentPath, storage, fs);
		}
		// if no separator occured in subspacePath, parent is storage root
		else if (!subspacePath.equals("")) {
			parent = new RemoteFile(storage, fs);
		}
		else {
			// subspacePath was "", so we have a storage root here
			parent = null;
		}
	}

	/**
	 * Constructor for a file inside the tree, where the parent file and the
	 * xfile are known. This constructor is used when the remote filechooser
	 * executes a ListDirectoryTask and returns a list of xfiles.
	 * 
	 * @param xfile
	 *            to be wrapped
	 *  
	 */
	public RemoteFile(XFile xfile, RemoteFile parent, RemoteFileSystem fs) {
		super(xfile.getName());
		this.storage = parent.getStorage();
		this.xfile = xfile;
		this.parent = parent;
		this.fs = fs;
		// cut plain filename from complete xfile path, e.g. simply "xyz.txt"
		String name = xfile.getName().substring(
				xfile.getName().lastIndexOf(separatorChar) + 1);
		// attach plain filename to parents path, e.g. "Home Home/tmp/bla" + "/"
		// + "xyz.txt"
		path = parent.getPath() + separatorChar + name;
	}

	/**
	 * @return true if xfile is readable
	 */
	public boolean canRead() {
		return xfile.canRead();
	}

	/**
	 * @return true if xfile is writable
	 */
	public boolean canWrite() {
		return xfile.canWrite();
	}

	/**
	 * Compare this to another file by comparing their paths lexicographically
	 * 
	 * @param pathname
	 * @return 0 if files are equal
	 */
	public int compareTo(File pathname) {
		return getPath().compareTo(pathname.getPath());
	}

	/**
	 * Do nothing yet
	 * 
	 * @return true
	 * @exception IOException
	 */
	public boolean createNewFile() throws IOException {
		return true;
	}

	/**
	 * Do nothing yet
	 * 
	 * @return true
	 */
	public boolean delete() {
		return true;
	}

	/**
	 * Do nothing yet
	 */
	public void deleteOnExit() {
	}

	public boolean equals(Object object) {
		RemoteFile file2 = (RemoteFile) object;
		String path1 = getPath();
		while (path1.startsWith(separatorChar)) {
			path1 = path1.substring(1);
		}
		while (path1.endsWith(separatorChar)) {
			path1 = path1.substring(0, path1.length() - 1);
		}
		String path2 = file2.getPath();
		while (path2.startsWith(separatorChar)) {
			path2 = path2.substring(1);
		}
		while (path2.endsWith(separatorChar)) {
			path2 = path2.substring(0, path2.length() - 1);
		}
		return file2.getStorage().equals(getStorage())
				&& file2.getVsite().equals(getVsite()) && path1.equals(path2);
	}

	/**
	 * @return true if xfile exists
	 */
	public boolean exists() {
		return xfile != null;
	}

	/**
	 * Get file object that represents this file with an absolute path. Since we
	 * always have an absolute path, we return "this".
	 * 
	 * @return always "this"
	 */
	public File getAbsoluteFile() {
		return this;
	}

	/**
	 * Get the absolute path for this file, which is equal to getPath() for the
	 * moment.
	 * 
	 * @return path as String
	 */
	public String getAbsolutePath() {
		return getPath();
	}

	/**
	 * Return instance itself.
	 * 
	 * @return always "this"
	 * @exception IOException
	 */
	public File getCanonicalFile() throws IOException {
		return this;
	}

	/**
	 * Identical to getPath(), as it is needed be the file chooser.
	 * 
	 * @return path as String
	 */
	public String getCanonicalPath() {
		return getPath();
	}

	/**
	 * Get the path from the wrapped XFile. This is the complete path as it
	 * appears on the remote file system.
	 * 
	 * @return path as String
	 */
	public String getCompletePath() {
		return xfile.getName();
	}

	public RemoteFileSystem getFileSystem() {
		return fs;
	}

	/**
	 * Cut the plain filename from the path and return it. e.g: "honk.txt" from
	 * "Home Absolute/tmp/honk.txt"
	 * 
	 * @return plain filename
	 */
	public String getName() {
		int index = getPath().lastIndexOf(separatorChar);
		if (index < 0) {
			return storage.toString();
		}
		else {
			return path.substring(index + 1);
		}
	}

	/**
	 * Get the path of this file's parent as String
	 * 
	 * @return parent's path
	 */
	public String getParent() {
		return getParentFile().getPath();
	}

	/**
	 * Get this file's parent object
	 * 
	 * @return The ParentFile value
	 */
	public File getParentFile() {
		return parent;
	}

	/**
	 * Get path to this file, e.g. "Home Absolute/tmp/honk.txt"
	 * 
	 * @return path as String
	 */
	public String getPath() {
		return path;
	}

	/**
	 * Get the Storage from this file
	 * 
	 * @return Storage
	 */
	public Storage getStorage() {
		return storage;
	}

	/**
	 * Get the path that the storage subspace represents. e.g: "Home Home" ->
	 * "/home/username"
	 * 
	 * @return path as String
	 */
	public String getStoragePath() {
		String complete = getCompletePath();
		String subpath = getSubspacePath();
		int index = complete.length() - subpath.length();
		String storagePath = complete.substring(0, index);
		// remove trailing "/"'s
		while (storagePath.endsWith(separatorChar)) {
			storagePath = storagePath.substring(0, storagePath.length() - 1);
		}
		return storagePath;
	}

	/**
	 * Get filename in storage subspace e.g: "Home/tmp/honk.txt" ->
	 * "tmp/honk.txt"
	 * 
	 * @return path as String
	 */
	public String getSubspacePath() {
		if (fs.isRoot(this)) {
			return "";
		}
		if (subspacePath == null) {
			subspacePath = parent.getSubspacePath() + separatorChar + getName();
			// remove leading "/"'s
			while (subspacePath.startsWith(separatorChar)) {
				subspacePath = subspacePath.substring(1);
			}
		}
		return subspacePath;
	}

	public Vsite getVsite() {
		return fs.getVsite();
	}

	public XFile getXFile() {
		return xfile;
	}

	public int hashCode() {
		int result = 17;
		result = 37 * result + getStorage().hashCode();
		result = 37 * result + getVsite().hashCode();
		String path1 = getPath();
		while (path1.startsWith(separatorChar)) {
			path1 = path1.substring(1);
		}
		while (path1.endsWith(separatorChar)) {
			path1 = path1.substring(0, path1.length() - 1);
		}
		result = 37 * result + path1.hashCode();
		return result;
	}

	/**
	 * Check if we have an absolute path. This is always true for the moment.
	 * 
	 * @return always true
	 */
	public boolean isAbsolute() {
		return true;
	}

	/**
	 * @return true if xfile is an XDirectory
	 */
	public boolean isDirectory() {
		return isRoot() || (xfile instanceof XDirectory);
	}

	/**
	 * @return true if xfile is not an XDirectory
	 */
	public boolean isFile() {
		return !isDirectory();
	}

	/**
	 * We do not hide files so far, because xfiles do not hav this flag.
	 * 
	 * @return always false
	 */
	public boolean isHidden() {
		return false;
	}

	/**
	 * Gets the root attribute of the RemoteFile object
	 * 
	 * @return The root value
	 */
	public boolean isRoot() {
		return fs.isRoot(this);
	}

	/**
	 * Get modified time stamp from xfile
	 * 
	 * @return xfile time stamp
	 */
	public long lastModified() {
		return xfile.getModifyDate().getTime();
	}

	/**
	 * @return xfile size
	 */
	public long length() {
		return xfile.getSize();
	}

	/**
	 * @return array containing complete paths of this directory
	 *  
	 */
	public String[] list() {
		if (!isDirectory()) {
			return null;
		}
		File[] files = listFiles();
		String[] listing = new String[files.length];
		for (int i = 0; i < files.length; i++) {
			listing[i] = ((RemoteFile) files[i]).getCompletePath();
		}
		return listing;
	}

	public String[] list(FilenameFilter filter) {
		String names[] = list();
		if ((names == null) || (filter == null)) {
			return names;
		}
		ArrayList v = new ArrayList();
		for (int i = 0; i < names.length; i++) {
			if (filter.accept(this, names[i])) {
				v.add(names[i]);
			}
		}
		return (String[]) (v.toArray(new String[0]));
	}

	/**
	 * @return directory contents as file array
	 */
	public File[] listFiles() {
		if (!isDirectory()) {
			return null;
		}
		return fs.getListing(this);
	}

	/**
	 * Do nothing yet
	 * 
	 * @return null
	 */
	public File[] listFiles(FileFilter filter) {
		return (File[]) null;
	}

	/**
	 * Do nothing yet
	 * 
	 * @return null
	 */
	public File[] listFiles(FilenameFilter filter) {
		return (File[]) null;
	}

	/**
	 * Do nothing yet
	 * 
	 * @return true
	 */
	public boolean mkdir() {
		logger.warning("mkdir in RemoteFile class not yet implemented!");
		return true;
	}

	/**
	 * Do nothing yet
	 * 
	 * @return true
	 */
	public boolean mkdirs() {
		if (exists()) {
			return false;
		}
		logger.warning("mkdirs in RemoteFile class not yet implemented!");
		return true;
	}

	/**
	 * Not used yet readObject is called to restore this filename. The original
	 * separator character is read. If it is different than the separator
	 * character on this system, then the old seperator is replaced by the local
	 * separator.
	 *  
	 */
	private synchronized void readObject(java.io.ObjectInputStream s)
			throws IOException, ClassNotFoundException {
	}

	/**
	 * Do nothing yet
	 * 
	 * @return true
	 */
	public boolean renameTo(File dest) {
		return true;
	}

	/**
	 * Set xfile's modified time stamp
	 * 
	 * @param time
	 *            The new LastModified value
	 * @return true
	 */
	public boolean setLastModified(long time) {
		if (time < 0) {
			throw new IllegalArgumentException(res.getString("NEGATIVE_TIME"));
		}
		xfile.setModifyDate(new Date(time));
		return true;
	}

	/**
	 * Set xfile to read only
	 * 
	 * @return always true
	 */
	public boolean setReadOnly() {
		xfile.setCanRead(false);
		return true;
	}

	/**
	 * Sets the xFile attribute of the RemoteFile object
	 * 
	 * @param xfile
	 *            The new xFile value
	 */
	public void setXFile(XFile xfile) {
		this.xfile = xfile;
	}

	/**
	 * Return path as String representation
	 * 
	 * @return getPath()
	 */
	public String toString() {
		return getPath();
	}

	public void updatePath() {
		fs.updatePath(this);
	}

	/**
	 * Not used yet
	 *  
	 */
	private synchronized void writeObject(java.io.ObjectOutputStream s)
			throws IOException {
		s.defaultWriteObject();
		//  s.writeChar(this.separatorChar); // Add the separator character
	}
}