/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.connection;

import java.io.IOException;
import java.net.URL;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.unicore.User;
import org.unicore.Vsite;
import com.pallas.unicore.extensions.Usite;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * Keep connestions alive in a Hashtable
 * 
 * @author Thomas Kentemich
 * @author Ralf Ratering
 * @version $Id: ConnectionManager.java,v 1.1 2004/05/25 14:58:51 rmenday Exp $
 */
public class ConnectionManager {

	private static Logger logger = Logger
			.getLogger("com.pallas.unicore.connection");
	private Hashtable connections = new Hashtable();

	/**
	 * Default Constructor
	 */
	public ConnectionManager() {
	}

	/**
	 * Get Connection for a User at a Vsite.
	 * 
	 * @param vsite
	 *            Destination Vsite
	 * @param user
	 *            User
	 * @return Cached or new Connection
	 * @throws Exception
	 */
	public Connection lookupConnection(Vsite vsite, User user) throws Exception {
		logger.info("Lookup Connection for vsite: " + vsite.getName()
				+ " Reference: " + vsite.getReference().getAddress());
		Connection connection = (Connection) connections.get(ResourceManager
				.getUniqueVsiteIdentifier(vsite));
		if (connection != null) {
			// if user has changed throw away connection
			if (!connection.getUser().equals(user)) {
				removeConnection(vsite);
				connection = null;
			}
		}
		if (connection == null) {
			// new connection
			if (vsite.getType().getDescription().equals("GS")) {
				//			    connection = new GSConnection(vsite.getAddress(), user);
			}
			else {
				connection = new Connection(vsite.getAddress(), user);
			}
			connections.put(ResourceManager.getUniqueVsiteIdentifier(vsite),
					connection);
			logger.info("New connection for vsite: " + vsite.getName()
					+ " user: " + user.getCertificate().getSubjectDN()
					+ " xlogin: " + user.getXlogin() + " project: "
					+ user.getProject());
		}
		logger.info("Connection for vsite: " + vsite.getName() + " user: "
				+ user.getCertificate().getSubjectDN() + " xlogin: "
				+ user.getXlogin() + " project: " + user.getProject());
		// reset connection values
		connection.setDirectOutput(null);
		connection.setIsAscii(false);
		return connection;
	}

	/**
	 * Get connection for a User at a Usite
	 * 
	 * @param usite
	 *            Usite
	 * @param user
	 *            User
	 * @return new or cached Connection
	 * @throws Exception
	 */
	public Connection lookupConnection(Usite usite, User user) throws Exception {
		Connection connection = (Connection) connections.get(usite);
		if (connection != null) {
			// if user has changed throw away connection
			if (!connection.getUser().equals(user)) {
				removeConnection(connection);
				connection = null;
			}
		}
		if (connection == null) {
			URL url = usite.getAddress();
			connection = new Connection(url.getProtocol() + "://"
					+ url.getHost() + ":" + url.getPort(), user);
			connections.put(usite, connection);
			logger.fine("New connection for usite: " + usite.getName()
					+ " user: " + user.getCertificate().getSubjectDN()
					+ " xlogin: " + user.getXlogin() + " project: "
					+ user.getProject());
		}
		logger.fine("Connection for usite: " + usite.getName() + " user: "
				+ user.getCertificate().getSubjectDN() + " xlogin: "
				+ user.getXlogin() + " project: " + user.getProject());
		return connection;
	}

	/**
	 * Remove Connection to a vsite
	 * 
	 * @param vsite
	 *            destination vsite
	 */
	public void removeConnection(Vsite vsite) {
		logger.fine("Removing previous connection to " + vsite);
		Connection connection = (Connection) connections.get(ResourceManager
				.getUniqueVsiteIdentifier(vsite));
		connections.remove(ResourceManager.getUniqueVsiteIdentifier(vsite));
		try {
			connection.close();
		}
		catch (IOException e) {
			logger.log(Level.WARNING, "Could not close removed connection.", e);
		}
	}

	/**
	 * Remove all cached connections
	 */
	public void removeAllConnections() {
		Enumeration connEnum = connections.elements();
		while (connEnum.hasMoreElements()) {
			Connection connection = (Connection) connEnum.nextElement();
			logger.fine("Removing previous connection for user "
					+ connection.getUser());
			try {
				connection.close();
			}
			catch (IOException e) {
				logger.log(Level.INFO, "Could not close connection: "
						+ connection, e);
			}
		}
		connections.clear();
	}

	/**
	 * Remove a specific Connection from cache
	 * 
	 * @param connection
	 *            to be removed
	 */
	private void removeConnection(Connection connection) {
		logger.fine("Removing previous connection for user "
				+ connection.getUser().getCertificate().getSubjectDN());
		try {
			connection.close();
		}
		catch (IOException e) {
			logger.log(Level.INFO, "Could not close connection: " + connection,
					e);
		}
		connections.remove(connection);
	}
}