/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.connection;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.UnknownHostException;
import java.security.SecureRandom;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocket;
import org.unicore.User;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.security.KeyStoreManager;

/**
 * This class creates SSL sockets and tunnels a proxy server if neccessary. If
 * the proxy needs authentication it will pop up a dialog to ask for it.
 * 
 * @author Ralf Ratering
 * @version $Id: UnicoreSSLSocketFactory.java,v 1.3 2003/06/30 06:56:55 rrate
 *          Exp $
 */
public final class UnicoreSSLSocketFactory {

	private static Logger logger = Logger
			.getLogger("com.pallas.unicore.connection");
	// random number generator for ssl socket
	private SecureRandom secureRandom;
	// ssl context initialized for each keystore
	private SSLContext sslContext;

	public UnicoreSSLSocketFactory(User user) {
		KeyStoreManager manager = ResourceManager.getKeystoreManager();
		try {
			sslContext = manager.initializeSSLContext(ResourceManager
					.getSecureRandom(), user.getCertificate());
		}
		catch (Exception e) {
			logger.log(Level.SEVERE, "Cannot initialize SSLFactory for user:"
					+ user, e);
		}
	}

	/**
	 * Creates a SSLSocket
	 * 
	 * @param port
	 *            the target port
	 * @param host
	 *            host address
	 * @return a socket connected to the given target address and port.
	 * @exception ConnectException
	 * @exception IOException
	 */
	public Socket createSocket(InetAddress host, int port) throws IOException {
		// SSLSocket socket =
		// (SSLSocket)sslContext.getSocketFactory().createSocket(
		//                          host, port);
		SSLSocket socket = (SSLSocket) sslContext.getSocketFactory()
				.createSocket();
		socket.connect(new InetSocketAddress(host, port), ResourceManager
				.getUserDefaults().getSocketTimeout());
		//        socket.setSoLinger(false, 0);
		return socket;
	}

	/**
	 * Creates a SSLSocket by address String for convenience.
	 * 
	 * @param address
	 *            something like "http://linus.pallas.de:8080"
	 * @return a socket connected to the given target address and port.
	 */
	public Socket createSocket(String address) throws IOException,
			UnknownHostException {
		int port = getPortFromAddress(address);
		InetAddress host = getHostFromAddress(address);
		return createSocket(host, port);
	}

	/**
	 * Try to tunnel a proxy server and ask for authentication if neccessary.
	 * 
	 * @param tunnel
	 *            a plain socket to the proxy server
	 * @param message
	 * @return Proxy server handshake reply as String
	 * @throws IOException
	 *             if anything goes wrong.
	 */
	private String doTunnelHandshake(Socket tunnel, String message)
			throws IOException {
		OutputStream out = tunnel.getOutputStream();
		logger.info("Sending now message via proxy: " + message);
		byte[] b;
		try {
			// We really do want ASCII7 -- the http protocol doesn't change
			// with locale.
			b = message.getBytes("ASCII7");
		}
		catch (UnsupportedEncodingException ignored) {
			// If ASCII7 isn't there, something serious is wrong, but
			// Paranoia Is Good (tm)
			b = message.getBytes();
		}
		out.write(b);
		out.flush();
		logger.info("...sending finished.");
		// We need to store the reply so we can create a detailed
		// error message to the user.
		byte[] reply = new byte[200];
		int replyLen = 0;
		int newlinesSeen = 0;
		boolean headerDone = false;
		// Done on first newline
		InputStream in = tunnel.getInputStream();
		boolean error = false;
		while (newlinesSeen < 2) {
			int i = in.read();
			if (i < 0) {
				throw new IOException("Unexpected EOF from proxy");
			}
			if (i == '\n') {
				headerDone = true;
				++newlinesSeen;
			}
			else if (i != '\r') {
				newlinesSeen = 0;
				if (!headerDone && replyLen < reply.length) {
					reply[replyLen++] = (byte) i;
				}
			}
		}
		return new String(reply);
	}

	/**
	 * Little helper method to get the host from a http address String
	 * 
	 * @param address
	 *            something like "http://linus.pallas.de:8080"
	 * @return host address extracted from complete http String
	 * @exception UnknownHostException
	 */
	private InetAddress getHostFromAddress(String address)
			throws UnknownHostException {
		int pos = address.indexOf(":") + 3;
		int pos2 = address.indexOf(":", pos);
		return InetAddress.getByName(address.substring(pos, pos2));
	}

	/**
	 * Little helper method to get the port from a http address String
	 * 
	 * @param address
	 *            something like "http://linus.pallas.de:8080"
	 * @return port extracted from address
	 * @throws NumberFormatException
	 *             if parsing failed
	 */
	private int getPortFromAddress(String address) throws NumberFormatException {
		int pos = address.indexOf(":") + 3;
		int pos2 = address.indexOf(":", pos);
		String host = address.substring(pos, pos2);
		String portString = address.substring(pos2 + 1);
		return Integer.parseInt(portString);
	}

	/**
	 * Gets the sslContext attribute of the UnicoreSSLSocketFactory object
	 * 
	 * @return The sslContext value
	 */
	public SSLContext getSslContext() {
		return this.sslContext;
	}

	/**
	 * Sets the DefaultSSLSocketFactory attribute of the UnicoreSSLSocketFactory
	 * object
	 */
	public void setDefaultSSLSocketFactory() {
		// set this initialized SSLContext as default for https connections
		HttpsURLConnection.setDefaultSSLSocketFactory(sslContext
				.getSocketFactory());
	}

	/**
	 * Provides UnicoreSSLSocketFactory with a <code>SecureRandom</code>
	 * instance. If one decides that the default instance,
	 * <code>java.security.SecureRandom</code>, does not sufficiently
	 * generate random numbers, one can provide an alternative, possibly "more
	 * random" implementation.
	 * 
	 * @param secureRandom
	 *            The new SecureRandom value
	 */
	public void setSecureRandom(SecureRandom secureRandom) {
		this.secureRandom = secureRandom;
	}

	public void setSslContext(SSLContext sslContext) {
		this.sslContext = sslContext;
	}
}