/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.extensions;

import java.awt.datatransfer.DataFlavor;
import java.io.File;
import java.util.StringTokenizer;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.unicore.ajo.DeclarePortfolio;

/**
 * Class representing a file import from a specified location into uspace.
 * 
 * @author Ralf Ratering
 * @version $Id: FileImport.java,v 1.1 2004/05/25 14:58:45 rmenday Exp $
 */
public class FileImport extends FileStorage {

	private static DataFlavor flavor;
	private static Logger logger = Logger
			.getLogger("com.pallas.unicore.extensions");
	final static long serialVersionUID = -1883378006618693063L;
	/**
	 * declare a portfolio containing the file/directory imported from nspace
	 */
	private transient DeclarePortfolio nspacePortfolio;
	/**
	 * remember actual destination if import is done from Nspace via hidden
	 * directory
	 */
	private String hiddenDestination = null;

	/**
	 * Get flavor for edit operations
	 * 
	 * @return DataFlavor
	 */
	public static DataFlavor getFlavor() {
		if (flavor == null) {
			try {
				flavor = new DataFlavor(Class
						.forName("com.pallas.unicore.extensions.FileImport"),
						"FileImport");
			}
			catch (ClassNotFoundException e) {
				logger
						.log(
								Level.SEVERE,
								"Error getting DataFlavore for com.pallas.unicore.extensions.FileImport",
								e);
			}
		}
		return flavor;
	}

	/**
	 * Constructor
	 * 
	 * @param storageItem
	 *            object describing source location
	 * @param sourceName
	 *            name of source file
	 * @param destinationName
	 *            name of destination file
	 */
	public FileImport(Object storageItem, String sourceName,
			String destinationName) {
		super(storageItem, sourceName, destinationName);
		// mask any blanks in destination name
		this.destinationName = maskBlanks(destinationName);
	}

	/**
	 * Constructor
	 * 
	 * @param storageItem
	 *            object describing destination location
	 * @param sourceName
	 *            name of source file
	 * @param destinationName
	 *            name of destination file
	 * @param overwrite
	 *            overwrite policy
	 */
	public FileImport(Object storageItem, String sourceName,
			String destinationName, boolean overwrite) {
		this(storageItem, sourceName, destinationName);
		this.overwrite = overwrite;
	}

	/**
	 * Constructor
	 * 
	 * @param storageItem
	 *            object describing destination location
	 * @param sourceName
	 *            name of source file
	 * @param destinationName
	 *            name of destination file
	 * @param overwrite
	 *            overwrite policy
	 * @param isAscii
	 *            binary or ASCII format
	 */
	public FileImport(Object storageItem, String sourceName,
			String destinationName, boolean overwrite, boolean isAscii) {
		this(storageItem, sourceName, destinationName);
		this.overwrite = overwrite;
		this.isAscii = isAscii;
	}

	/**
	 * Get separator of destination file system (always UNIX)
	 * 
	 * @return separator string
	 */
	public String getDestinationFileSeparator() {
		return "/";
	}

	/**
	 * @return DeclarePortfolio defining a portfolio needed to import files from
	 *         nspace
	 */
	public DeclarePortfolio getNspacePortfolio() {
		return nspacePortfolio;
	}

	/**
	 * Get separator of source file system (Local)
	 * 
	 * @return separator string
	 */
	public String getSourceFileSeparator() {
		if (this.isLocal()) {
			return File.separator;
		}
		// remote file systems are assumed to be UNIX
		else {
			return "/";
		}
	}

	/**
	 * Gets the base name in the source name
	 * 
	 * @return The base name value of the source name
	 */
	public String getSourceCanonicalBasename() {
		String basename;
		String sourceName = new String(this.sourceName);
		// Use File class for local files which handles Windows and UNIX file
		// separators transparently (e.g. in case of job defined on UNIX Client
		// which is now submitted on Windows Client)
		// Use canonical path to resolve all '.' and '..' pathes as well as
		// wildcards
		// which resolve to a single path
		if (this.isLocal()) {
			File file = new File(sourceName);
			try {
				sourceName = file.getCanonicalPath();
			}
			catch (Exception ex) {
				// this will happen if wildcards resolve to multiple paths
				// choose the best guess
				sourceName = file.getAbsolutePath();
			}
		}
		if (sourceName.endsWith(getSourceFileSeparator())) {
			basename = sourceName.substring(0, sourceName.length() - 1);
		}
		else {
			basename = sourceName;
		}
		basename = basename.substring(basename
				.lastIndexOf(getSourceFileSeparator()) + 1);
		return basename;
	}

	/**
	 * Get flavor of sub class
	 * 
	 * @return sub class flavor
	 */
	public DataFlavor getSubClassFlavor() {
		return getFlavor();
	}

	/**
	 * Mask blanks in Windows filenames with backslashes to be compatible with
	 * the UNIX Uspace
	 */
	private String maskBlanks(String filename) {
		if (filename == null) {
			return null;
		}
		String result = "";
		StringTokenizer tokenizer = new StringTokenizer(filename, " ");
		while (tokenizer.hasMoreTokens()) {
			String token = tokenizer.nextToken();
			result = result + token;
			if (token.endsWith("\\")) {
				result += " ";
				continue;
			}
			int index = filename.indexOf(token) + token.length();
			String remainder = filename.substring(index);
			while (!token.endsWith("\\") && remainder.startsWith(" ")) {
				result += "\\ ";
				index++;
				remainder = filename.substring(index);
			}
		}
		if (result.endsWith("\\ ")) {
			result = result.substring(0, result.length() - 2);
		}
		return result;
	}

	/**
	 * Set destination filename
	 * 
	 * @param destinationName
	 *            destination filename
	 */
	public void setDestinationName(String destinationName) {
		maskBlanks(destinationName);
		super.setDestinationName(destinationName);
	}

	/**
	 * Set DeclarePortfolio task
	 * 
	 * @param nspacePortfolio
	 *            DeclarePortfolio task
	 */
	public void setNspacePortfolio(DeclarePortfolio nspacePortfolio) {
		this.nspacePortfolio = nspacePortfolio;
	}

	/**
	 * To resolve all kinds of imports with renaming, a file/directory from
	 * Local is first streamed without renaming to a temporary hidden directory
	 * (set in the following as the hidden destination name of the FileImport at
	 * hand). A dependent hidden CopyFile (additionally specified here) will
	 * then copy the streamed file/directory to its actual destination. The
	 * renaming will take place implicitly due to the use of CopyFile.
	 * 
	 * Set the hidden destination name (a hidden directory in case of wildcards)
	 * 
	 * @param hiddenDestination
	 *            the hidden destination name
	 */
	public void setHiddenDestination(String hiddenDestination) {
		this.hiddenDestination = hiddenDestination;
	}

	/**
	 * @return name of the hidden destination file/directory
	 */
	public String getHiddenDestination() {
		return hiddenDestination;
	}

	/**
	 * Overwrite method from super class
	 * 
	 * @return String describing operation
	 */
	public String toString() {
		return storageItem + ": " + sourceName + " -> " + destinationName;
	}
}