/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.extensions;

import java.io.Serializable;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.unicore.Identifier;
import org.unicore.Vsite;
import org.unicore.resources.CapacityResource;
import org.unicore.resources.Context;
import org.unicore.resources.Home;
import org.unicore.resources.Memory;
import org.unicore.resources.Node;
import org.unicore.resources.PerformanceResource;
import org.unicore.resources.Priority;
import org.unicore.resources.PriorityValue;
import org.unicore.resources.Processor;
import org.unicore.resources.Resource;
import org.unicore.resources.RunTime;
import org.unicore.resources.SoftwareResource;
import org.unicore.resources.Storage;
import org.unicore.resources.StorageServer;
import org.unicore.resources.TextInfoResource;
import org.unicore.resources.USpace;
import org.unicore.sets.ResourceEnumeration;
import org.unicore.sets.ResourceSet;
import com.pallas.unicore.client.tables.NamedObject;
import com.pallas.unicore.container.errorspec.ErrorSet;
import com.pallas.unicore.container.errorspec.UError;

/**
 * Class encapsulating a ResourceSet to manage loading, saving and editing
 * 
 * @author Ralf Ratering
 * @version $Id: NamedResourceSet.java,v 1.1 2004/05/25 14:58:45 rmenday Exp $
 */
public class NamedResourceSet implements Serializable, Cloneable, NamedObject {

	public final static Context DECISION_CONTEXT = new Context("",
			"MakeReturnCodeDecision", "");
	protected static Logger logger = Logger
			.getLogger("com.pallas.unicore.extensions");
	final static long serialVersionUID = 4464281534699904799L;
	public final static String SITE_DEFAULTS = "Site Defaults";

	/**
	 * Static method to generate an empty default set from scratch
	 * 
	 * @return a NamedResourceSet containing Memory, Node Processor and RunTime
	 *         resources
	 */
	public static NamedResourceSet getEmptyResourceSet() {
		TextInfoResource textInfo = new TextInfoResource("Default Resource",
				"",
				"No resources have been loaded from virtual site. Using default resources");
		Memory memory = new Memory("Memory", Float.MAX_VALUE, 0.0, 0.0);
		Node nodes = new Node("Nodes", Float.MAX_VALUE, 0.0, 0.0);
		Processor processor = new Processor("Processors per Node",
				Float.MAX_VALUE, 0.0, 0.0);
		RunTime runTime = new RunTime("CPU Time per Job", Float.MAX_VALUE, 0.0,
				0.0);
		ResourceSet resourceSet = new ResourceSet();
		resourceSet.add(textInfo);
		resourceSet.add(memory);
		resourceSet.add(nodes);
		resourceSet.add(processor);
		resourceSet.add(runTime);
		NamedResourceSet namedSet = new NamedResourceSet("Empty Set",
				resourceSet);
		return namedSet;
	}

	public static void main(String args[]) {
		NamedResourceSet resourceSet = new NamedResourceSet("Honk");
		resourceSet.add(new SoftwareResource("Bla1", "CPMD1", "1.0"));
		resourceSet.add(new SoftwareResource("Bla2", "CPMD2", "2.0"));
		resourceSet.add(new SoftwareResource("Bla3", "CPMD3", "3.0"));
		resourceSet.add(new SoftwareResource("Bla4", "CPMD4", "4.0"));
		System.err.println(resourceSet.toStringDetailed());
		System.err.println("REMOVING...");
		resourceSet.removeAllElements();
		System.err.println(resourceSet.toStringDetailed());
	}
	private String name, filename;
	private ResourceSet resourceSet;
	private Vsite vsite;

	public NamedResourceSet() {
		this(SITE_DEFAULTS);
	}

	public NamedResourceSet(String name) {
		this(name, new ResourceSet());
	}

	public NamedResourceSet(String name, ResourceSet resourceSet) {
		this.name = name;
		setResourceSet(resourceSet);
	}

	/**
	 * Adds a resource to the encapsulated ResourceSet. If there are resources
	 * of the same type already in the resource set they will be removed.
	 * 
	 * @param resource
	 *            resource to be added
	 */
	public void add(Resource resource) {
		// remove all resources ogfsame type from set
		Resource alreadyIn = findResource(resource);
		while (alreadyIn != null) {
			resourceSet.remove(alreadyIn);
			alreadyIn = findResource(resource);
		}
		resourceSet.add(resource);
	}

	public Vector addApplicationResources(Vector result) {
		return addResourcesByName("org.unicore.resources.Application", result);
	}

	public Vector addCapabilityResources(Vector result) {
		return addResourcesByName("org.unicore.resources.CapabilityResource",
				result);
	}

	public Vector addCapacityResources(Vector result) {
		ResourceEnumeration resourceEnum = resourceSet.elements();
		while (resourceEnum.hasMoreElements()) {
			Resource resource = resourceEnum.nextElement();
			if (resource instanceof CapacityResource
					&& !(resource instanceof PerformanceResource)) {
				result.add(resource);
			}
		}
		return result;
	}

	public Vector addContextResources(Vector result) {
		return addResourcesByName("org.unicore.resources.Context", result);
	}

	public void addErrorsToSet(ErrorSet errorSet, Identifier taskId,
			Vsite vsite, NamedResourceSet siteResources) {
		ResourceEnumeration resourceEnum = resourceSet.elements();
		while (resourceEnum.hasMoreElements()) {
			Resource resource = (Resource) resourceEnum.nextElement();
			Resource siteResource = siteResources.findResource(resource);
			if (siteResource == null) {
				errorSet.add(new UError(taskId, "Requested resource <"
						+ resource + ">" + " not available on " + vsite));
			}
			else if (resource instanceof CapacityResource) {
				CapacityResource capacity = (CapacityResource) resource;
				CapacityResource siteCapacity = (CapacityResource) siteResource;
				double request = capacity.getRequest();
				double min = siteCapacity.getMinRequest();
				double max = siteCapacity.getMaxRequest();
				if (request < min || request > max) {
					errorSet.add(new UError(taskId, "Requested " + capacity
							+ ": <" + (int) request + " " + capacity.getUnits()
							+ " not available on " + vsite));
				}
			}
		}
	}

	/**
	 * @deprecated use getHome, because since AJO4.0 there is only one home
	 *             resource
	 */
	public Vector addHomeResources(Vector result) {
		return addResourcesByName("org.unicore.resources.Home", result);
	}

	public Vector addInformationResources(Vector result) {
		return addResourcesByName("org.unicore.resources.InformationResource",
				result);
	}

	public Vector addPerformanceResources(Vector result) {
		return addResourcesByName("org.unicore.resources.PerformanceResource",
				result);
	}

	public Vector addPriorityResources(Vector result) {
		return addResourcesByName("org.unicore.resources.Priority", result);
	}

	private Vector addResourcesByName(String className, Vector result) {
		ResourceEnumeration resourceEnum = resourceSet.elements();
		while (resourceEnum.hasMoreElements()) {
			Resource resource = resourceEnum.nextElement();
			try {
				if (Class.forName(className).isInstance(resource)) {
					result.add(resource);
				}
			}
			catch (ClassNotFoundException e) {
				logger.log(Level.SEVERE,
						"Error adding Resource ro NamedResourceSet "
								+ className, e);
			}
		}
		return result;
	}

	public Vector addSoftwareResources(Vector result) {
		return addResourcesByName("org.unicore.resources.SoftwareResource",
				result);
	}

	public Vector addStorageResources(Vector result) {
		return addResourcesByName("org.unicore.resources.Storage", result);
	}

	public Vector addStorageServerResources(Vector result) {
		return addResourcesByName("org.unicore.resources.StorageServer", result);
	}

	public Object clone() {
		try {
			NamedResourceSet clone = (NamedResourceSet) super.clone();
			clone.setResourceSet(resourceSet.cloneSet());
			return clone;
		}
		catch (CloneNotSupportedException e) {
			logger.log(Level.SEVERE, "Cloning " + this + " failed.", e);
			return null;
		}
	}

	public boolean contains(Resource resource) {
		return (!(findResource(resource) == null));
	}

	public boolean containsStorageResources() {
		ResourceEnumeration resourceEnum = resourceSet.elements();
		while (resourceEnum.hasMoreElements()) {
			if (resourceEnum.nextElement() instanceof Storage) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Try to find a resource of a specific type in the resource set. For
	 * storage resources also the subspace values have to match. For priorities
	 * the priority values are compared.
	 * 
	 * @param toBeFound
	 *            Resource to be found
	 * @return The matching resource from the resource set
	 */
	public Resource findResource(Resource toBeFound) {
		ResourceEnumeration resourceEnum = resourceSet.elements();
		while (resourceEnum.hasMoreElements()) {
			Resource resource = resourceEnum.nextElement();
			// cannot use object equals here, because CapacityResources
			// are only equal when min/max/etc. are equal
			if (resource.getClass().equals(toBeFound.getClass())) {
				// for storages compare subspaces
				if (resource instanceof Storage) {
					Storage storage = (Storage) resource;
					Storage toBeFoundStorage = (Storage) toBeFound;
					// compare names of storage servers
					if (storage instanceof StorageServer) {
						StorageServer server = (StorageServer) storage;
						StorageServer toBeFoundServer = (StorageServer) toBeFoundStorage;
						if (server.getName().equals(toBeFoundServer.getName())) {
							return storage;
						}
					}
					else {
						return storage;
					}
				}
				// for priorities compare values
				else if (resource instanceof Priority) {
					Priority priority = (Priority) resource;
					Priority toBeFoundPriority = (Priority) toBeFound;
					if (toBeFoundPriority.getValue()
							.equals(priority.getValue())) {
						return priority;
					}
				}
				// for software resources use equals to compare name and version
				else if (resource instanceof SoftwareResource) {
					if (resource.equals(toBeFound)) {
						return resource;
					}
				}
				else {
					return resource;
				}
			}
		}
		// if we got here no resource was found
		return null;
	}

	public SoftwareResource findSoftwareResourceByName(String softwareName) {
		ResourceEnumeration resourceEnum = resourceSet.elements();
		while (resourceEnum.hasMoreElements()) {
			Resource resource = resourceEnum.nextElement();
			if (resource instanceof SoftwareResource) {
				SoftwareResource software = (SoftwareResource) resource;
				if (software.getName().equals(softwareName)) {
					return software;
				}
			}
		}
		return null;
	}

	public String getFilename() {
		return filename;
	}

	public Home getHome() {
		ResourceEnumeration resourceEnum = resourceSet.elements();
		while (resourceEnum.hasMoreElements()) {
			Resource resource = resourceEnum.nextElement();
			if (resource instanceof Home) {
				return (Home) resource;
			}
		}
		return null;
	}

	public String getName() {
		return name;
	}

	public PriorityValue getPriorityValue() {
		ResourceEnumeration resourceEnum = resourceSet.elements();
		while (resourceEnum.hasMoreElements()) {
			Resource resource = resourceEnum.nextElement();
			if (resource instanceof Priority) {
				Priority priority = (Priority) resource;
				return priority.getValue();
			}
		}
		return null;
	}

	/**
	 * Get the encapsulated resource set. CAUTION: The resource set may be
	 * referenced by multiple tasks, so if you modify the returned set, you may
	 * affect the resources of outher tasks. Please, check this carefully and
	 * use getResourceSetClone instead!!
	 * 
	 * @return the encasulated resource set
	 */
	public ResourceSet getResourceSet() {
		return resourceSet;
	}

	/**
	 * Get a clone of the encapsulated resource set. Use this method instead of
	 * the plain getResourceSet(), if you want to modify the set without
	 * affecting other tasks that may have a reference to it.
	 * 
	 * @return A clone of the encapsulated resource set
	 */
	public ResourceSet getResourceSetClone() {
		return resourceSet.cloneSet();
	}

	public ResourceSet getStorageResources() {
		ResourceSet storages = new ResourceSet();
		ResourceEnumeration resourceEnum = resourceSet.elements();
		while (resourceEnum.hasMoreElements()) {
			Resource resource = resourceEnum.nextElement();
			if (resource instanceof Storage) {
				storages.add(resource);
			}
		}
		return storages;
	}

	public USpace getUspace() {
		ResourceEnumeration resourceEnum = resourceSet.elements();
		while (resourceEnum.hasMoreElements()) {
			Resource resource = resourceEnum.nextElement();
			if (resource instanceof USpace) {
				return (USpace) resource;
			}
		}
		return null;
	}

	public USpace getUspaceHome() {
		return getUspace();
	}

	public Vsite getVsite() {
		return vsite;
	}

	public boolean isCorrect() {
		return true;
	}

	/**
	 * Check if the resource set is empty
	 *  
	 */
	public boolean isEmpty() {
		return !resourceSet.elements().hasMoreElements();
	}

	/**
	 * If resource set is empty and name equals SITE_DEFAULTS we have a site
	 * default resource set.
	 * 
	 *  
	 */
	public boolean isSiteDefault() {
		return name.equals(SITE_DEFAULTS);
	}

	public boolean matchesSiteResources(NamedResourceSet siteResources) {
		ResourceEnumeration resourceEnum = resourceSet.elements();
		while (resourceEnum.hasMoreElements()) {
			Resource resource = (Resource) resourceEnum.nextElement();
			Resource siteResource = siteResources.findResource(resource);
			if (siteResource == null) {
				return false;
			}
			if (resource instanceof CapacityResource) {
				CapacityResource capacity = (CapacityResource) resource;
				CapacityResource siteCapacity = (CapacityResource) siteResource;
				double request = capacity.getRequest();
				double min = siteCapacity.getMinRequest();
				double max = siteCapacity.getMaxRequest();
				if (request < min || request > max) {
					return false;
				}
			}
		}
		// if we got here resources match
		return true;
	}

	public void removeAllElements() {
		Vector tmpVec = new Vector();
		ResourceEnumeration resourceEnum = resourceSet.elements();
		while (resourceEnum.hasMoreElements()) {
			tmpVec.add(resourceEnum.nextElement());
		}
		for (int i = 0; i < tmpVec.size(); i++) {
			resourceSet.remove((Resource) tmpVec.elementAt(i));
		}
	}

	public void setFilename(String filename) {
		this.filename = filename;
	}

	public void setName(String name) {
		this.name = name;
	}

	public void setResourceSet(ResourceSet resourceSet) {
		this.resourceSet = resourceSet;
		// always add decision context to make ExecuteTasks produce Decision
		// file
		if (!resourceSet.isIn(DECISION_CONTEXT)) {
			resourceSet.add(DECISION_CONTEXT);
		}
	}

	public void setVsite(Vsite vsite) {
		this.vsite = vsite;
	}

	public String toString() {
		return name;
	}

	public String toStringDetailed() {
		String result = "NamedResourceSet " + name + "\n";
		ResourceEnumeration resourceEnum = resourceSet.elements();
		while (resourceEnum.hasMoreElements()) {
			Resource resource = (Resource) resourceEnum.nextElement();
			result += resource + ":" + resource.getDescription();
			if (resource instanceof CapacityResource) {
				CapacityResource capacity = (CapacityResource) resource;
				result += ", Request: " + capacity.getRequest();
				result += ", Min: " + capacity.getMinRequest();
				result += ", Max: " + capacity.getMaxRequest();
				result += ", Default: " + capacity.getDefaultRequest();
			}
			result += "\n";
		}
		return result;
	}
}