/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.extensions;

import java.util.Vector;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.GroupContainer;
import com.pallas.unicore.container.IfContainer;
import com.pallas.unicore.container.TaskContainer;
import com.pallas.unicore.container.errorspec.ErrorSet;
import com.pallas.unicore.container.errorspec.UError;

/**
 * Test a task for return codes or success.
 * 
 * @author Ralf Ratering
 * @version $Id: ReturnCodeTest.java,v 1.1 2004/05/25 14:58:45 rmenday Exp $
 */
public class ReturnCodeTest extends AbstractTest implements Cloneable {

	/**
	 * Return code equals
	 */
	public final static ReturnCodeTestType EQUAL = new ReturnCodeTestType(
			ReturnCodeTestType.EQUAL);
	/**
	 * Return code not equals
	 */
	public final static ReturnCodeTestType NOT_EQUAL = new ReturnCodeTestType(
			ReturnCodeTestType.NOT_EQUAL);
	/**
	 * Task successful
	 */
	public final static ReturnCodeTestType SUCCESSFUL = new ReturnCodeTestType(
			ReturnCodeTestType.SUCCESSFUL);
	/**
	 * Task not successful
	 */
	public final static ReturnCodeTestType NOT_SUCCESSFUL = new ReturnCodeTestType(
			ReturnCodeTestType.NOT_SUCCESSFUL);
	final static long serialVersionUID = -2638238924226364124L;
	private static Vector types;
	private ActionContainer addedFlag;
	private ActionContainer testContainer;
	private int value = 0;

	/**
	 * Get all test types
	 * 
	 * @return Vector containing all types
	 */
	public static Vector getTypes() {
		if (types == null) {
			types = new Vector();
			types.add(DISABLED);
			types.add(EQUAL);
			types.add(NOT_EQUAL);
			types.add(SUCCESSFUL);
			types.add(NOT_SUCCESSFUL);
		}
		return types;
	}

	/**
	 * check test parameters
	 * 
	 * @param errorSet
	 *            ErrorSet that errors will be added to
	 * @param container
	 *            container belonging to test
	 * @return updated error set
	 */
	public ErrorSet checkContents(ErrorSet errorSet, ActionContainer container) {
		super.checkContents(errorSet, container);
		if (!isEnabled()) {
			return errorSet;
		}
		if (testContainer == null) {
			errorSet.add(new UError(container.getIdentifier(),
					"Missing test task in return code test."));
			return errorSet;
		}
		if (!getType().equals(SUCCESSFUL) && !getType().equals(NOT_SUCCESSFUL)) {
			boolean error = false;
			if (!(testContainer instanceof TaskContainer)) {
				error = true;
			}
			else {
				TaskContainer task = (TaskContainer) testContainer;
				if (!task.hasExecuteTask()) {
					error = true;
				}
			}
			if (error) {
				errorSet
						.add(new UError(
								container.getIdentifier(),
								testContainer
										+ " in return code test, does not return a value."));
			}
		}
		if (!testContainer.getIgnoreFailure()) {
			testContainer.setIgnoreFailure(true);
			addedFlag = testContainer;
		}
		if (container instanceof IfContainer) {
			boolean error = true;
			GroupContainer groupContainer = container.getParentContainer();
			for (int i = 0; i < groupContainer.getDependencies().length; i++) {
				ActionContainer succ = groupContainer.getDependencies()[i]
						.getSucc();
				ActionContainer pred = groupContainer.getDependencies()[i]
						.getPred();
				if (pred.getIdentifier() == getTestContainer().getIdentifier()) {
					if (getTestContainer().isPredecessorOf(container)) {
						error = false;
					}
					break;
				}
			}
			if (error) {
				errorSet
						.add(new UError(
								container.getIdentifier(),
								testContainer
										+ " must be an ancestor of "
										+ container
										+ " in the dependency graph, because it will be tested for return code/success."));
			}
		}
		return errorSet;
	}

	/**
	 * If we added an ignore failure flag for this test, get the container
	 * 
	 * @return container where we added the flag
	 */
	public ActionContainer getAddedFlag() {
		return addedFlag;
	}

	/**
	 * Get the test container
	 * 
	 * @return container that is tested
	 */
	public ActionContainer getTestContainer() {
		return testContainer;
	}

	/**
	 * value to test for
	 * 
	 * @return value to test for
	 */
	public int getValue() {
		return value;
	}

	private boolean isEqual(ActionContainer container1,
			ActionContainer container2) {
		return container1.getIdentifier() == container2.getIdentifier();
	}

	/**
	 * Remove all artificially added flags
	 */
	public void removeAddedFlags() {
		if (addedFlag != null) {
			addedFlag.setIgnoreFailure(false);
			addedFlag = null;
		}
	}

	/**
	 * Set container where we added an ignore failure flag
	 * 
	 * @param addedFlag
	 *            the container
	 */
	public void setAddedFlag(ActionContainer addedFlag) {
		this.addedFlag = addedFlag;
	}

	/**
	 * Set the test container
	 * 
	 * @param testContainer
	 *            container that is tested
	 */
	public void setTestContainer(ActionContainer testContainer) {
		this.testContainer = testContainer;
	}

	/**
	 * Set the test value
	 * 
	 * @param value
	 *            test value
	 */
	public void setValue(int value) {
		this.value = value;
	}

	/**
	 * Overwrite super class method
	 * 
	 * @return String description
	 */
	public String toString() {
		return "Container: " + testContainer + ", Test: " + type + ", Value: "
				+ value;
	}
}