/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.extensions;

import java.net.URL;
import org.unicore.Unicore;

/**
 * Class representing a UNICORE USite
 * 
 * @author Thomas Kentemich
 * @author Ralf Ratering
 * @version $Id: Usite.java,v 1.1 2004/05/25 14:58:45 rmenday Exp $
 */
public class Usite extends Unicore implements Comparable {

	final static long serialVersionUID = 6965391670916531770L;
	private String name;
	private Type type;
	private URL url;

	public Usite(URL url, String name) {
		this(url, name, Type.CLASSIC);
	}

	public Usite(URL url, String name, Type type) {
		this.url = url;
		this.name = name;
		this.type = type;
	}

	public int compareTo(Object object) {
		Usite usite2 = (Usite) object;
		return (name.compareTo(usite2.getName()));
	}

	/**
	 * Compares two Usites. The result is <code>true</code> if and only if the
	 * name, URL host and port are equal.
	 * 
	 * @param obj
	 *            the Usite to compare against.
	 * @return <code>true</code> if the URL addresses are the same;
	 *         <code>false</code> otherwise.
	 */
	public boolean equals(Object obj) {
		if (!(obj instanceof Usite)) {
			return false;
		}
		Usite usite2 = (Usite) obj;
		return getAddress().equals(usite2.getAddress());
	}

	public URL getAddress() {
		return url;
	}

	public String getName() {
		return name;
	}

	public Type getType() {
		return type;
	}

	/**
	 * Always override hashCode when overriding equals.
	 *  
	 */
	public int hashCode() {
		int result = 17;
		result = 37 * result + getName().hashCode();
		return result;
	}

	public void setAddress(URL url) {
		this.url = url;
	}

	public void setName(String name) {
		this.name = name;
	}

	public void setType(Type type) {
		this.type = type;
	}

	public String toString() {
		return name;
	}

	public static class Type extends Unicore {

		public final static Type CLASSIC = new Type("Classical UNICORE Site");
		public final static Type REGISTRY = new Type("Grid Service Registry");
		final static long serialVersionUID = -7208106932332931313L;
		private String description;

		/**
		 * Create a new Type
		 */
		public Type() {
			this.description = CLASSIC.getDescription();
		}

		/**
		 * Create a new Type.
		 * <p>
		 * 
		 * Note that this is publically available so that the range of supported
		 * types can be extended by agreement between the clients and sites.
		 *  
		 */
		public Type(String description) {
			this.description = description;
		}

		/**
		 * Types are equal if their descriptions are equal.
		 * <p>
		 * 
		 * Need to do carefully as these cross JVM vboundaries through
		 * serialisation and so "==" will fail even on statics.
		 *  
		 */
		public boolean equals(Object o) {
			try {
				return (o.getClass() == getClass() && getDescription().equals(
						((Type) o).getDescription()));
			}
			catch (NullPointerException nex) {
				return false;
			}
		}

		public String getDescription() {
			return description;
		}

		public int hashCode() {
			return getDescription().hashCode();
		}
	}
}