/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.requests;

import java.io.File;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Vector;
import java.util.logging.Level;
import org.unicore.AJOIdentifier;
import org.unicore.Vsite;
import org.unicore.ajo.ActionGroup;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcomeAck;
import org.unicore.upl.UnicoreResponse;
import com.pallas.unicore.client.trees.ContainerNode;
import com.pallas.unicore.client.trees.PanelNode;
import com.pallas.unicore.connection.Connection;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.container.TaskContainer;
import com.pallas.unicore.extensions.FileExport;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.security.JobConverter;
import com.pallas.unicore.utility.DeleteAll;

/**
 * Delete a job node from job monitor
 * 
 * @author Thomas Kentemich
 * @version $Id: DeleteJob.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public class DeleteJob extends ObservableNodeThread {

	private JobContainer jobContainer;
	private UnicoreResponse response = new UnicoreResponse(0, "<None>");

	/**
	 * Constructor
	 * 
	 * @param node
	 *            job node in job monitor to be deleted
	 */
	public DeleteJob(ContainerNode node) {
		super(node);
		jobContainer = (JobContainer) node.getActionContainer();
	}

	/**
	 * Delete all spooled files belonging to this job We currently do not
	 * support spooled files that persist after the job is deleted
	 * 
	 * @param top
	 *            top level job container
	 */
	private void deleteSpooledFiles(JobContainer top) {
		Vector newNspaceTransfers = top.getNspaceExportTasks();
		Vector subJobs = top.getSubJobs();
		// delete sub job spool files
		for (int j = 0; j < subJobs.size(); j++) {
			deleteSpooledFiles((JobContainer) subJobs.elementAt(j));
		}
		if (newNspaceTransfers.size() == 0) {
			return;
		}
		Vector fileExports = new Vector();
		for (int i = 0; i < newNspaceTransfers.size(); i++) {
			TaskContainer ac = (TaskContainer) newNspaceTransfers.elementAt(i);
			Vector spooledFiles = ac.getSpooledFiles();
			if (spooledFiles != null) {
				for (int j = 0; j < spooledFiles.size(); j++) {
					fileExports.add((FileExport) spooledFiles.elementAt(j));
				}
			}
		}
		DeleteSpoolFiles delete = new DeleteSpoolFiles(top, fileExports);
		delete.start();
	}

	private void deleteTempFiles() {
		if (jobContainer.getLastUpdateUPLDirectory() != null) {
			try {
				DeleteAll.deleteAll(new File(Connection.getSessionDir()
						+ File.separator
						+ jobContainer.getLastUpdateUPLDirectory()));
			}
			catch (IOException e) {
				logger.log(Level.WARNING,
						"Deleting temporary directories failed.", e);
			}
		}
		ActionGroup actionGroup = jobContainer.getActionGroup();
		if (actionGroup == null) {
			return;
		}
		findActionGroups(actionGroup);
	}

	/**
	 * Find an ActionGroup directory and delete it
	 * 
	 * @param actionGroup
	 *            containing ExecuteTask
	 */
	private void findActionGroups(ActionGroup actionGroup) {
		Enumeration actions = actionGroup.getActions();
		while (actions.hasMoreElements()) {
			org.unicore.ajo.AbstractAction nextAction = (org.unicore.ajo.AbstractAction) (actions
					.nextElement());
			if (nextAction instanceof ActionGroup) {
				// find action group directories
				String sessionDir = Connection.getSessionDir() + File.separator
						+ nextAction.getUPLDirectoryName();
				try {
					logger.info("Deleting: " + sessionDir);
					DeleteAll.deleteAll(new File(sessionDir));
				}
				catch (IOException e) {
					logger.log(Level.SEVERE, "Deleting " + sessionDir
							+ " failed.", e);
				}
			}
		}
	}

	/**
	 * Method will be called from start() in Thread
	 */
	public void run() {
		try {
			jobContainer = (JobContainer) getNode().getUserObject();
			PanelNode vsiteNode = (PanelNode) getNode().getParent();
			Vsite vsite = null;
			// if we have a vsite node it's safest to use the underlying vsite
			// object
			if (vsiteNode != null) {
				vsite = (Vsite) vsiteNode.getUserObject();
			}
			// service jobs won't have vsite nodes, so take the container's
			// vsite
			else {
				vsite = jobContainer.getVsite();
			}
			// delete the job by acknowleging the outcome
			RetrieveOutcomeAck roAck = new RetrieveOutcomeAck(
					(AJOIdentifier) jobContainer.getIdentifier(), vsite);
			JobConverter.convert(roAck, true);
			Connection connection = ResourceManager.getConnectionManager()
					.lookupConnection(vsite, ResourceManager.getUser(vsite));
			Reply roAckReply = connection.sendMessage(roAck, null);
			int returnCode = roAckReply.getLastEntry().getReturnCode();
			response = roAckReply.getLastEntry();
			logger.info("Job deleted: " + jobContainer.getName());
			if (returnCode != 0) {
				logger.severe("DeleteJob Error: " + response.getReturnCode()
						+ " " + response.getComment());
				notifyObservers(this, response);
				return;
			}
			deleteSpooledFiles(jobContainer);
			// Delete all tmp files in the Session directory
			deleteTempFiles();
		}
		catch (Exception e) {
			logger.log(Level.SEVERE, "Executing delete job failed.", e);
		}
		notifyObservers(this, response);
	}
}