/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.requests;

import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;
import java.util.Hashtable;
import java.util.Vector;
import java.util.logging.Level;
import org.unicore.User;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.ListDirectory;
import org.unicore.outcome.AbstractJob_Outcome;
import org.unicore.outcome.ListDirectory_Outcome;
import org.unicore.outcome.Outcome;
import org.unicore.outcome.XFile;
import org.unicore.resources.Storage;
import org.unicore.sets.ResourceSet;
import org.unicore.sets.XFileEnumeration;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcomeReply;
import org.unicore.utility.ConsignForm;
import com.pallas.unicore.client.remotefilechooser.RemoteFile;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * Get listings for a list of remote directories in one job.
 * 
 * @author Ralf Ratering
 * @version $Id: GetListings.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public class GetListings extends ObservableRequestThread {

	private Vector dirNodes;
	private Vector dirs;
	private Vsite vsite;

	/**
	 * Constructor
	 * 
	 * @param dirs
	 *            directories to get listings for
	 * @param dirNodes
	 *            nodes from file chooser that belong to directories
	 * @param vsite
	 *            destination vsite
	 */
	public GetListings(Vector dirs, Vector dirNodes, Vsite vsite) {
		this.dirs = dirs;
		this.dirNodes = dirNodes;
		this.vsite = vsite;
	}

	/**
	 * Constructor
	 * 
	 * @param dirs
	 *            directories to get listings for
	 * @param vsite
	 *            destination vsite
	 */
	public GetListings(Vector dirs, Vsite vsite) {
		this(dirs, null, vsite);
	}

	/**
	 * Get directories
	 * 
	 * @return directoties
	 */
	public Vector getDirectories() {
		return dirs;
	}

	/**
	 * Get directory nodes
	 * 
	 * @return directory nodes
	 */
	public Vector getDirectoryNodes() {
		return dirNodes;
	}

	/**
	 * Start the thread
	 */
	public void run() {
		Hashtable table = new Hashtable();
		User user = ResourceManager.getUser(vsite);
		AbstractJob job = new AbstractJob(ResourceManager.getServicePrefix()
				+ "_GetListingsJob_"
				+ ResourceManager.getNextObjectIdentifier(), null, null, vsite,
				user);
		Vector listDirs = new Vector();
		for (int i = 0; i < dirs.size(); i++) {
			RemoteFile dir = (RemoteFile) dirs.elementAt(i);
			String dirname = dir.getSubspacePath();
			Storage storage = dir.getStorage();
			ListDirectory listDir = new ListDirectory("ListDirectory");
			listDir.setResources(new ResourceSet(storage));
			listDir.setTarget(dirname);
			listDirs.add(listDir);
			job.add(listDir);
			logger.fine("Getting listing for " + dirname + " at storage "
					+ storage);
		}
		Reply reply = nonPolling(job, vsite, user);
		// now analyze outcome
		if (!(reply instanceof RetrieveOutcomeReply)) {
			logger.warning("Virtual site not available: " + vsite
					+ "\nReason: " + reply.getLastEntry().getComment());
			return;
		}
		else {
			try {
				RetrieveOutcomeReply roReply = (RetrieveOutcomeReply) reply;
				// deserialize the Outcome Object
				byte[] outcomeBytes = roReply.getOutcome();
				Outcome outcome = (Outcome) new ObjectInputStream(
						new ByteArrayInputStream(outcomeBytes)).readObject();
				AbstractJob_Outcome ajOut = ConsignForm
						.convertFrom((AbstractJob_Outcome) outcome);
				for (int i = 0; i < listDirs.size(); i++) {
					ListDirectory listDir = (ListDirectory) listDirs
							.elementAt(i);
					Outcome ajOutcome = ajOut.getOutcome(listDir.getId());
					if (ajOutcome instanceof ListDirectory_Outcome) {
						ListDirectory_Outcome listDirOutcome = (ListDirectory_Outcome) ajOutcome;
						Vector listing = new Vector();
						RemoteFile parent = (RemoteFile) dirs.elementAt(i);
						if (listDirOutcome.getListing() == null) {
							logger.severe("Could not get listing for "
									+ listDir + " at site " + vsite);
						}
						else {
							XFileEnumeration fileEnum = listDirOutcome
									.getListing().elements();
							while (fileEnum.hasMoreElements()) {
								XFile xfile = (XFile) fileEnum.nextElement();
								RemoteFile file = new RemoteFile(xfile, parent,
										parent.getFileSystem());
								listing.add(file);
							}
						}
						table.put(parent, listing);
					}
				}
			}
			catch (Exception e) {
				logger.log(Level.SEVERE, "Could not get directory listing ", e);
			}
		}
		// notify all my observers on task end
		notifyObservers(this, table);
	}
}