/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.requests;

import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.util.logging.Level;
import org.unicore.AAIdentifier;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractAction;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.GetActionStatus;
import org.unicore.outcome.AbstractActionStatus;
import org.unicore.outcome.AbstractJob_Outcome;
import org.unicore.outcome.GetActionStatus_Outcome;
import org.unicore.outcome.Outcome;
import org.unicore.sets.OutcomeEnumeration;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcomeReply;
import org.unicore.utility.ConsignForm;
import com.pallas.unicore.client.UserDefaults;
import com.pallas.unicore.client.trees.ContainerNode;
import com.pallas.unicore.client.trees.PanelNode;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.ExecuteContainer;
import com.pallas.unicore.container.GroupContainer;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.UserMessages;

/**
 * Get status for a container node in the job monitor
 * 
 * @author Thomas Kentemich
 * @author Ralf Ratering
 * @version $Id: GetNewJobStatus.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public class GetNewJobStatus extends ObservableNodeThread {

	private Hashtable serialOutcomes = new Hashtable();
	private UserDefaults userDefaults = ResourceManager.getUserDefaults();

	/**
	 * Constructor
	 * 
	 * @param node
	 *            container node in job monitor
	 */
	public GetNewJobStatus(ContainerNode node) {
		super(node);
	}

	/**
	 * Build the AJO
	 */
	private AbstractJob buildJobStatusRequest(JobContainer jobContainer,
			Vsite vsite) {
		if (jobContainer.getStatus() != null
				&& jobContainer.getStatus().isEquivalent(
						AbstractActionStatus.DONE)) {
			return null;
		}
		AbstractJob serviceJob = new AbstractJob(servicePrefix
				+ "_GetJobStatus" + ResourceManager.getNextObjectIdentifier(),
				null, null, vsite, ResourceManager.getUser(vsite));
		buildTaskStatusRequest(jobContainer, serviceJob, vsite);
		return serviceJob;
	}

	/**
	 * Add task for an ActionContainer
	 */
	private void buildTaskStatusRequest(ActionContainer container,
			AbstractJob serviceJob, Vsite vsite) {
		if (container.getStatus() != null
				&& container.getStatus()
						.isEquivalent(AbstractActionStatus.DONE)) {
			return;
		}
		AbstractAction significantTask = container.getActionGroup();
		if (container instanceof ExecuteContainer) {
			ExecuteContainer execCont = (ExecuteContainer) container;
			significantTask = execCont.getExecuteTask();
		}
		if (significantTask != null) {
			GetActionStatus getStatus = new GetActionStatus("GetStatus"
					+ ResourceManager.getNextObjectIdentifier(), null,
					significantTask.getId());
			logger.info("Building status request for task: " + significantTask
					+ " in container " + container);
			serviceJob.add(getStatus);
		}
		if (container instanceof GroupContainer) {
			GroupContainer groupContainer = (GroupContainer) container;
			Vector tasks = groupContainer.getTasks();
			for (int i = 0; i < tasks.size(); i++) {
				ActionContainer task = (ActionContainer) tasks.elementAt(i);
				if (task instanceof JobContainer) {
					JobContainer job = (JobContainer) task;
					AbstractJob subJob = buildJobStatusRequest(job, job
							.getVsite());
					if (subJob != null) {
						serviceJob.add(subJob);
					}
				}
				else {
					buildTaskStatusRequest(task, serviceJob, vsite);
				}
			}
		}
	}

	/**
	 * Start the Thread
	 */
	public void run() {
		ContainerNode jobNode = (ContainerNode) getNode();
		jobNode.setState(ContainerNode.STATE_UPDATING);
		JobContainer jobContainer = (JobContainer) jobNode.getActionContainer();
		PanelNode vsiteNode = (PanelNode) jobNode.getParent();
		Vsite vsite = null;
		// if we have a vsite node it's safest to use the underlying vsite
		// object
		if (vsiteNode != null) {
			vsite = (Vsite) vsiteNode.getUserObject();
		}
		// service jobs won't have vsite nodes, so take the container's vsite
		else {
			vsite = jobContainer.getVsite();
		}
		AbstractJob statusRequest = buildJobStatusRequest(jobContainer, vsite);
		if (statusRequest == null) {
			notifyObservers(this, jobNode);
			return;
		}
		try {
			Reply reply = nonPolling(statusRequest, vsite, ResourceManager
					.getUser(vsite));
			if (reply == null || !(reply instanceof RetrieveOutcomeReply)) {
				throw new Exception("Could not contact virtual site " + vsite);
			}
			else {
				RetrieveOutcomeReply roReply = (RetrieveOutcomeReply) reply;
				// deserialize the Outcome Object
				byte[] serializedOutcome = roReply.getOutcome();
				if (serializedOutcome != null) {
					Outcome outcome = (Outcome) new ObjectInputStream(
							new ByteArrayInputStream(serializedOutcome))
							.readObject();
					if (outcome instanceof AbstractJob_Outcome) {
						serializeOutcome(ConsignForm
								.convertFrom((AbstractJob_Outcome) outcome));
						Enumeration enum = serialOutcomes.elements();
						while (enum.hasMoreElements()) {
							GetActionStatus_Outcome next = (GetActionStatus_Outcome) enum
									.nextElement();
							spreadStatusByContainer(next, jobContainer);
						}
					}
					else {
						throw new Exception("Unexpected type of outcome");
					}
				}
			}
		}
		catch (Exception e) {
			UserMessages.warning("Updating status for job " + jobContainer
					+ " failed.");
			logger.log(Level.SEVERE, "Could not get new job statut for job "
					+ jobContainer, e);
		}
		notifyObservers(this, jobNode);
	}

	private void serializeOutcome(AbstractJob_Outcome ajOutcome) {
		OutcomeEnumeration oen = ajOutcome.getOutcomes();
		while (oen.hasMoreElements()) {
			Outcome next = oen.nextElement();
			if (next instanceof GetActionStatus_Outcome) {
				if (((GetActionStatus_Outcome) next).getActionIdentifier() != null) {
					serialOutcomes.put(((GetActionStatus_Outcome) next)
							.getActionIdentifier(),
							((GetActionStatus_Outcome) next));
				}
				else {
					logger
							.severe("GetActionStatus failed for: "
									+ ((GetActionStatus_Outcome) next).getId()
											.getName() + " Reason: "
									+ next.getReason());
				}
			}
			else if (next instanceof AbstractJob_Outcome) {
				serializeOutcome((AbstractJob_Outcome) next);
			}
		}
	}

	private void spreadStatusByContainer(GetActionStatus_Outcome statusOutcome,
			ActionContainer container) {
		AAIdentifier id = container.getIdentifier();
		if (container instanceof ExecuteContainer) {
			ExecuteContainer execCont = (ExecuteContainer) container;
			if (execCont.getExecuteTask() == null) {
				// ExecuteContainer does not contain an ExecuteTask?
				// -> wrong implementation, get out
				return;
			}
			id = execCont.getExecuteTask().getId();
		}
		if (statusOutcome.getActionIdentifier().equals(id)) {
			container.setStatus(statusOutcome.getActionStatus());
		}
		if (container instanceof GroupContainer) {
			GroupContainer groupContainer = (GroupContainer) container;
			Vector tasks = groupContainer.getTasks();
			for (int i = 0; i < tasks.size(); i++) {
				ActionContainer task = (ActionContainer) tasks.elementAt(i);
				spreadStatusByContainer(statusOutcome, task);
			}
		}
	}
}