package org.unicore;

import java.util.Date;
import java.util.Random;

/**
 * An Identifier is used to <i>uniquely</i> identify the 
 * containing Object across different JVMs. 
 * <p>
 * It is used within the AJO classes
 * to replace the default Java object identity (as this only lasts for one instance
 * of a JVM and will fail to identify a Job passed from, say, a JPA
 * to an NJS and then returned as the same entity).
 * <p>
 * We must be careful to override all methods that work on Object
 * identity and use the Java identifier. Thus, 
 * Objects that contain an <tt>Identifier</tt> should override 
 * <tt>equals</tt> and <tt>hashCode</tt> to work on the stored value.
 * These properties mean that <code>Identifier</code> should store
 * a 32-bit value.
 * <p>
 * It is assigned a unique value at creation time and this should remain 
 * unchanged from then onwards.
 * <p>
 * This implementation uses a sequence based on the time that the JPA
 * was started.
 *
 * @see org.unicore.ajo.Portfolio
 * @see org.unicore.ajo.AbstractAction
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 1 
 *
 * @version Identifier.java 1.6 05/13/02 
 * 
 **/
public abstract class Identifier extends Unicore {

    // A value unique(ish) for each Java VM invocation
    private static int base;
    private static int getAndIncBase() {return base++;}

    // This Object's identifier
    private int value;
    /**
     * Return the unique value of this Identifier
     *
     **/
    public int getValue() {return value;}

    static {
        // This uses all 64 bits to set a base, which I guess 
        // is better than using the last 32 bits of the time.
        base = (new Random( (new Date()).getTime() )).nextInt();
    }
        
    protected Identifier() {
        this("");
    }

    /**
     * Create a new Identifier with a unique value.
     *
     * @param name The name of the Identifier (convenience only)
     *
     **/
    protected Identifier(String name) {
        this.value = getAndIncBase();
        this.name = name;
    }

    /**
     * Create a new Identifier with the given value.
     *
     * @param name The name of the Identifier (convenience only).
     * @param value The value for this Identifier.
     *
     **/
    protected Identifier(String name, int value) {
        this.value = value;
        this.name = name;
    }

    /**
     * Test for equality using the value of the Identifier.
     *<p>
     * This implementation ensures that Identifiers will
     * evaluate as equals if their value is the same, even
     * if they have passed across JVM boundaries (e.g. by
     * serialisation).
     *
     **/
    public final boolean equals(Object in) {
        if(in == null) return false;
        return( in.getClass() ==  this.getClass() &&
                ((Identifier)in).value == value);
    }

    /**
     * Compute a new value for this Identifier.
     *<p>
     * Following the execution of this method the Identifier
     * will no longer be equals to the Identifiers containing
     * its previous value.
     *<p>
     * This is used
     * when AJOs are put into consign form so that
     * reuse of "old" instances does not cause re-consigns
     * of the same identities.
     *
     * @see org.unicore.utility.ConsignForm
     *
     **/
    public void changeIdentity() {
        value = getAndIncBase();
    }

    /**
     * Return a hash code based on the value
     * of the Identifier.
     *
     **/
    public final int hashCode() {return value;}

    private String name;
    /**
     * Return the name of this Identifier.
     *
     **/
    public final String getName() {return name;}

    /**
     * Return the name of this Identifier.
     * <p>
     * The name of the Identifier is provided as a convenience to
     * the creator of an AJO. It has no effect on the identity
     * of the object containing the Identifier instance.
     *
     **/
    public final void setName(String new_name) {name = new_name;}

    static final long serialVersionUID = 4804028480255040640L;

}
//
// Change history:
//          15 Mar 2002 SvdB V4, no arg constructor
//          20 Dec 2000 SvdB Removed clone
//          24 Nov 2000 SvdB Improved comments
//           4 May 2000 SvdB V3, change package, make hierarchy, protect,
//                           added changeIdentity() 
//           2 Mar 1999 SvdB Cloneable 
//           3 Feb 1999 SvdB Access to base synchronized, expose value 
//           7 Dec 1998 SvdB Second release to Unicore 
//          12 Jun 1998 SvdB Initial release to Unicore 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
