package org.unicore.ajo;

import org.unicore.sets.ResourceSet;

/**
 * Copy a file or directory.
 * <p>
 * If the source ("from" file) is a directory, then its contents are copied recursively.
 * <p>
 * If the "copy from Uspace" flag is true, then the from file is in the Uspace
 * of the AJO (possibly in a subdirectory as determined by the path field of 
 * the {@link org.unicore.resources.USpace}) found in the CopyFile's resources.
 * The "to" file is found in the other instance of a {@link org.unicore.resources.Storage}
 * found in the CopyFile's resources. The default for both resources is the root of
 * the Uspace.
 * <p>
 * If the "copy from Uspace" flag is false, then the roles are reversed.
 * <p>
 * The default action is to copy <em>to</em> the Uspace (i.e. "copy from Uspace" flag is false)
 *
 * @see org.unicore.outcome.CopyFile_Outcome
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3
 *
 * @version CopyFile.java 1.10 09/18/02 
 * 
 **/
public final class CopyFile extends FileAction {

    static final long serialVersionUID = -4038599585044115217L;

    /**
     * Create a new CopyFile.
     * <p>
     * @param name The name of the CopyFile
     * @param extra_information Client supplied extra information about the CopyFile
     * @param resources Resources needed to execute the CopyFile
     * @param from The name of the file or directory to copy
     * @param to The destination file or directory name
     * @param overwrite Whether the copy should overwrite any existing files
     * @param from_uspace Is Uspace the source of the files?
     *
     **/
    public CopyFile( String name,
                       byte[] extra_information,
                       ResourceSet resources,
                       String from,
                       String to,
                       boolean overwrite,
                       boolean from_uspace
                     ) {

        super(name,extra_information,resources);

        this.from = from;
        this.to = to;
        this.overwrite = overwrite;
        this.from_uspace = from_uspace;
    }

    /**
     * Create a new CopyFile.
     * <p>
     * @param name The name of the CopyFile
     * @param resources Resources needed to execute the CopyFile
     * @param from The name of the file or directory to copy
     * @param to The destination file or directory name
     * @param overwrite Whether the copy should overwrite any existing files
     * @param from_uspace Is Uspace the source of the files?
     *
     **/
    public CopyFile( String name,
                       ResourceSet resources,
                       String from,
                       String to,
                       boolean overwrite,
                       boolean from_uspace
                     ) {

        this(name,(byte[]) null, resources, from, to, overwrite, from_uspace);
    }

    /**
     * Create a new CopyFile.
     * <p>
     * @param name The name of the CopyFile
     *
     **/
    public CopyFile( String name) {
        this(name,(byte[]) null,(ResourceSet) null, (String) null, (String) null, false, false);
    }

    public CopyFile() {
        this("");
    }

    private String from;
    /**
     * Return the source file or directory name for the copy.
     *
     **/
    public String getFrom() {return from;}

    /**
     * Set the source file or directory name for the copy.
     *
     **/
    public void setFrom(String from) {this.from = from;}

    private String to;
    /**
     * Return the destination file or directory for the copy.
     *
     **/
    public String getTo() {return to;}

    /**
     * Set the destination file or directory for the copy.
     * <p>
     * If the "to" file is null (or empty string), then the destination
     * file or directory will have the same name as the source and be placed
     * in the directory implied by the CopyFile's resources.
     *
     **/
    public void setTo(String to) {this.to = to;}

    boolean overwrite;
    /**
     * Overwrite existing destination file?
     * <p>
     * If no overwriting is allowed (this method returns false), then the task will
     * fail if the copy would overwrite an existing file. If the "to" file is a
     * directory, then the test will be applied to each file in the directory (that
     * is, writing files into existing directories is always allowed).
     * <p>
     * Overwriting existing files or writing into existing directories is always
     * subject to permissions on the existing files or directories and the task
     * will fail if there is no suitable write permission.
     *
     **/
    public boolean willOverwrite() {return overwrite;}

    /**
     * Set overwrite policy.
     *
     **/
    public void setOverwrite(boolean overwrite) {this.overwrite = overwrite;}

    boolean from_uspace;
    /**
     * Copy from Uspace to the other {@link org.unicore.resources.Storage}?
     * <p>
     * If this is false, then the file is copied to Uspace.
     *
     * @since AJO 4.0
     *
     **/
    public boolean copyFromUspace() {return from_uspace;}

    /**
     * Set direction of the copy.
     *
     * @since AJO 4.0
     *
     **/
    public void setCopyFromUspace(boolean from_uspace) {this.from_uspace = from_uspace;}

}
//
// Change history:
//    13 Sep 2002 SvdB Changed comments re overwrites 
//    18 Mar 2002 SvdB V4 no arg constructor, added from Uspace flag
//     9 Jun 2000 SvdB Renamed from Copy
//     5 May 2000 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
