package org.unicore.ajo;

import org.unicore.sets.ResourceSet;
import org.unicore.sets.OptionSet;
import org.unicore.sets.PortfolioList;

/**
 * An ExecuteTask represents something that will be incarnated
 * to an executable file.
 * <p>
 * The incarnation of an ExecuteTask will be done by giving the name
 * of a file to execute. ExecuteTasks can therefore take command line
 * arguments, read their parent shell's environment, write to stdout
 * and stderr and may read from stdin.
 * <p>
 * ExecuteTask does not contain a field to represent the executable
 * and so the NJS must supply one during execution e.g. to run a
 * package or a site specific command.
 * <p>
 * Standard out and standard error from the executed script 
 * (which may include commands added by the NJS as well as the
 * incarnation of the ExecuteTask) are returned in
 * {@link org.unicore.outcome.ExecuteTask_Outcome}. However,
 * the user can choose to redirect the stdout and stderr
 * from the incarnated task to files for further processing
 * in the AJO (note that this redirection does not include
 * any output from commands added by the NJS).
 * <p>
 * The Outcome of an ExecuteTask can contain a {@link org.unicore.outcome.Decision}
 *
 * @see org.unicore.outcome.ExecuteTask_Outcome
 * @see If
 * @see RepeatGroup
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 1
 *
 * @version ExecuteTask.java 1.9 09/09/02 
 * 
 **/
public class ExecuteTask extends AbstractTask implements Decidable {

    static final long serialVersionUID = -1416827979811138285L;

    /**
     * Create a new ExecuteTask.
     * <p>
     * @param name The name of the ExecuteTask
     * @param extra_information Client supplied extra information about the ExecuteTask
     * @param resources Resources needed to execute the ExecuteTask
     * @param  environment_variables Name/values to set in the executable's environment
     * @param command_line Seamful additions to the exectable's command line
     * @param stdin File to write to stdin
     * @param redirect_stdout Stdout redirection policy
     * @param redirect_stderr Stderr redirection policy
     * @param verbose Return executed script
     * @param version Return exectable version information
     *
     **/
    public ExecuteTask( String name,
                        byte[] extra_information,
                        ResourceSet resource,
                        OptionSet environment_variables,
                        String command_line,
                        Portfolio stdin,
                        boolean redirect_stdout,
                        boolean redirect_stderr,
                        boolean verbose,
                        boolean version
                      ) {

        super(name,extra_information,resource);

        this.environment_variables = environment_variables;
        this.command_line = command_line;
        this.stdin = stdin;
        this.redirect_stdout = redirect_stdout;
        this.redirect_stderr = redirect_stderr;
        this.verbose = verbose;
        this.version = version;

    }

    /**
     * Create a new ExecuteTask.
     * <p>
     * @param name The name of the ExecuteTask
     * @param resources Resources needed to execute the ExecuteTask
     * @param  environment_variables Name/values to set in the executable's environment
     * @param command_line Seamful additions to the exectable's command line
     * @param stdin File to write to stdin
     * @param redirect_stdout Stdout redirection policy
     * @param redirect_stderr Stderr redirection policy
     * @param verbose Return executed script
     * @param version Return exectable version information
     *
     **/
    public ExecuteTask( String name,
                        ResourceSet resource,
                        OptionSet environment_variables,
                        String command_line,
                        Portfolio stdin,
                        boolean redirect_stdout,
                        boolean redirect_stderr,
                        boolean verbose,
                        boolean version
                      ) {

        this(name,(byte[])null,resource,environment_variables,command_line,stdin,redirect_stdout,redirect_stderr,verbose,version);
    }


    /**
     * Create a new ExecuteTask.
     * <p>
     * @param name The name of the ExecuteTask
     *
     **/
    public ExecuteTask(String name) {
        this(name,(byte[]) null,(ResourceSet) null,
              (OptionSet) null,
              (String) null, (Portfolio) null, false, false, false, false);
    }

    public ExecuteTask() {
        this("");
    }

    private OptionSet environment_variables;
    /**
     * Return the environment variables to set for the execution of the task.
     * <p>
     * The name/value pairs in this set are added to the shell
     * environment of the script that will execute the incarnated executable.
     * <p>
     * The interpretation of the String pairs is that the first 
     * is the environment variable name and the second is its value. 
     * <p>
     *
     **/
    public OptionSet getEnvironmentVariables() {return environment_variables;}

    /**
     * Set new environment variables. 
     * <p>
     * This overwrites previous values.
     *
     **/
    public void setEnvironmentVariables(OptionSet environment_variables) {
        this.environment_variables = environment_variables;
    }

    /**
     * Add a new environment variable.
     *
     **/
    public void addEnvironmentVariable(Option variable) {
        if(environment_variables == null) environment_variables = new OptionSet();
        if(!environment_variables.isIn(variable)) environment_variables.add(variable);
    }

    private String command_line;
    /**
     * Get the string to be added to the incarnated executable's command line.
     * <p>
     * This string can be used to supply (seamful) command line
     * arguments to exectables when the options are not catered for
     * by a seamless task definition in the AJO hierarchy.
     * <p>
     * It is added to the end of the incarnated command line.
     *
     **/
    public String getCommandLine() {return command_line;}

    /**
     * Set the string to be added to the incarnated executable's command line.
     *
     **/
    public void setCommandLine(String command_line) {
        this.command_line = command_line;
    }

    private Portfolio stdout;
    private boolean redirect_stdout;
    /**
     * Get the stdout redirection policy.
     *
     **/
    public boolean stdoutIsRedirected() {return redirect_stdout;}

    /**
     * Set the stdout redirection policy.
     *
     **/
    public void setRedirectStdout(boolean redirect_stdout) {
        this.redirect_stdout = redirect_stdout;
    }

    /**
     * Return the file to which stdout is saved.
     * <p>
     * If stdout redirection is set, then the returned Portfolio will 
     * incarnate to a single file
     * that contains the stdout produced by the execution
     * of this ExecuteTask.
     * <p>
     * If stdout redirection is not set, then this will return null.
     *
     **/
    public Portfolio getStdout() {
        if(stdoutIsRedirected()) {
            if(stdout == null)  stdout = new Portfolio("stdout");
            return stdout;
        }
        else {
            return (Portfolio) null;
        }
    }

    private String stdout_file_name;
    /**
     * Name the file to which stdout will be redirected.
     * <p>
     * Sets the stdout redirection policy to true.
     * <p>
     * This is optional. If it is not set, then the NJS will
     * create a name for the file which is unique within the
     * Uspace. If it is set, then it is the user's responsibility
     * to ensure that the file name is unique.
     *
     * @since AJO 3.2
     *
     **/
    public void setStdoutFileName(String name) {
        setRedirectStdout(true);
        stdout_file_name = name;
    }

    /**
     * Get the name for the stdout file.
     *
     **/
    public String getStdoutFileName() {return stdout_file_name;}

    private Portfolio stderr;
    private boolean redirect_stderr;
    /**
     * Get the stderr redirection policy.
     *
     **/
    public boolean stderrIsRedirected() {return redirect_stderr;}

    /**
     * Set the stderr redirection policy.
     *
     **/
    public void setRedirectStderr(boolean redirect_stderr) {
        this.redirect_stderr = redirect_stderr;
    }

    /**
     * Return the file to which stderr is saved.
     * <p>
     * If stderr redirection is set, then the returned Portfolio will 
     * incarnate to a single file
     * that contains the stderr produced by the execution
     * of this ExecuteTask.
     * <p>
     * If stderr redirection is not set, then this will return null.
     *
     **/
    public Portfolio getStderr() {
        if(stderrIsRedirected()) {
            if(stderr == null)  stderr = new Portfolio("stderr-"+getName());
            return stderr;
        }
        else {
            return (Portfolio) null;
        }
    }

    private String stderr_file_name;
    /**
     * Name the file to which stderr will be redirected.
     * <p>
     * Sets the stderr redirection policy to true.
     * <p>
     * This is optional. If it is not set, then the NJS will
     * create a name for the file which is unique within the
     * Uspace. If it is set, then it is the user's responsibility
     * to ensure that the file name is unique.
     *
     * @since AJO 3.2
     *
     **/
    public void setStderrFileName(String name) {
        setRedirectStderr(true);
        stderr_file_name = name;
    }

    /**
     * Get the name for the stderr file.
     *
     **/
    public String getStderrFileName() {return stderr_file_name;}

    private Portfolio stdin;
    /**
     * Set file to be directed to the stdin of the incarnated executable.
     * <p>
     * If the Portfolio is non-null then it must incarnate to a single file
     * which will be sent to the executable's stdin.
     *
     **/
    public Portfolio getStdin() {return stdin;}

    /**
     * Set file to be directed to the stdin of the incarnated executable.
     *
     **/
    public void setStdin(Portfolio stdin) {this.stdin = stdin;}

    private boolean verbose;
    /**
     * Set view of incarnated commands.
     * <p>
     * Does the user want to see the incarnated command?
     *
     **/
    public boolean isVerboseOn() {return verbose;}

    /**
     * Set view of incarnated commands.
     *
     **/
    public void setVerbose(boolean verbose) {this.verbose =  verbose;}

    private boolean version;
    /**
     * Get return of executable version information.
     * <p>
     * Does the user want to see the version information
     * of the executable used to incarnate the ExecuteTask?
     * If supported.
     *
     **/
    public boolean isVersionOn() {return version;}

    /**
     * Set return of executable version information.
     *
     **/
    public void setVersion(boolean version) {this.version =  version;}

}
//
// Change history:
//           8 Aug 2002 SvdB Decidable
//          18 Mar 2002 SvdB V4 no arg constructor
//          26 Sep 2001 SvdB Made a concrete class
//           2 Mar 2001 SvdB Added ability to name stdout and stderr files
//           5 May 2000 SvdB V3.0 
//          08 Feb 1999 SvdB made abstract 
//           7 Dec 1998 SvdB Second release to Unicore 
//           4 Nov 1998 SvdB Stdin can now be a list of Portfolios 
//          12 Jun 1998 SvdB Initial release to Unicore 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
