package org.unicore.ajo;

/**
 * If provides support for conditional execution of actions depending
 * on the result code of a {@link Decidable}.
 * <p>
 * The If action tests the {@link org.unicore.outcome.Decision} of the
 * test {@link Decidable} against the test value and executes the true branch if the Strings
 * are equal, otherwise it executes the false branch.
 * <p>
 * Both branches are AbstractActions, in practice they will be
 * ActionGroups or AbstractJobs.
 * <p>
 * The If will fail if the test action has not completed
 * execution or if the Decision is null.
 * 
 * @see Decidable
 * @see org.unicore.outcome.If_Outcome
 * @see org.unicore.outcome.ExecuteTask_Outcome#getResultCode
 *
 * @author D. Snelling (fecit)
 *
 * @since AJO 3.2
 *
 * @version If.java 1.6 09/09/02
 *
 **/
public final class If extends ConditionalAction {

    static final long serialVersionUID = -3479950854571165310L;

    /**
     * Create a new If action.
     * <p>
     * @param name The name of the If action
     * @param extra_information Client supplied extra information about the If action
     * @param test_task The Decidable whose decision is used in the expression.
     * @param test_value The test string
     * @param true_branch The action executed if the expression evaluated to true.
     * @param fail_branch The action executed if the expression evaluated to false.
     *
     **/
    public If (String name, 
               byte[] extra_information, 
               Decidable test_task,
               String test_value,
               AbstractAction true_branch,
               AbstractAction false_branch) {
        super(name,extra_information,true_branch,false_branch);
        this.test_task    = test_task;
        this.test_value   = test_value;
    }

    /**
     * Create a new If action.
     * <p>
     * @param name The name of the If action
     * @param test_task The Decidable whose decision is used in the expression.
     * @param test_value The test string
     * @param true_branch The action executed if the expression evaluated to true.
     * @param fail_branch The action executed if the expression evaluated to false.
     *
     **/
    public If (String name, 
               Decidable test_task,
               String test_value,
               AbstractAction true_branch,
               AbstractAction false_branch) {
        this(name, (byte[]) null, test_task, test_value, true_branch, false_branch);
    }

    /**
     * Create a new If action.
     * <p>
     * @param name The name of the If action
     *
     **/
    public If (String name) {
        this(name,(byte[]) null, (Decidable) null, (String)null, (AbstractAction) null, (AbstractAction) null);
    }

    public If () {
        this("");
    }

    private Decidable test_task;

    /**
     * Return the Decidable whose decision is to be tested.
     *
     **/
    public Decidable getTestTask() {return test_task;}

    /**
     * Set the test task. 
     *
     **/
    public void setTestTask (Decidable test_task) {this.test_task = test_task;}

    private String test_value;
    /**
     * @return the test value
     *
     **/
    public String getTest() {return test_value;}

    /**
     * Set the test value.  The test value can be null, if it is the true branch is taken.
     *
     **/
    public void setTest(String test_value) {this.test_value = test_value;}

}
//
// Change history:
//           8 Aug 2002 SvdB Uses Decidable
//          18 Mar 2002 SvdB V4 no arg constructor, other changes
//          31 Jan 2001 DFS Created
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
