package org.unicore.ajo;

import org.unicore.PortfolioIdentifier;

/**
 * A Portfolio represents a collection of files
 * that will be in the Uspace when the AJO is executed. However,
 * when the AJO is being prepared we do not know the names (or even number)
 * of these files. Portfolios are handles on these execute time files
 * and are used when preparing an AJO to show where the files used
 * by a task come from.
 * <p>
 * Portfolios are analagous to Java reference types. In the same
 * way that the actual class used by a reference class is not known
 * until the code is executed the  files that make up a Portfolio are
 * not known until the AJO is executed.
 * <p>
 * Portfolios are created when the execution of a Task produces
 * files in the Uspace. For example, {@link ImportTask} creates
 * a Portfolio to hold the files that will be put into  the Uspace
 * following the execution of the Task, and {@link CompileTask} creates
 * a Portfolio to hold the object files resulting from the compilation.
 * <p>
 * Portfolios are created in the current Uspace directory  (given by 
 * the path field of the {@link org.unicore.resources.USpace} of the task).
 * If a task uses a POrtffolio, then it must have been created in the
 * the current Uspace directory, Portfolios in other directories are not
 * available to the task.
 * <p>
 * In some cases files are created in the Uspace without the exection
 * of a task in the AJO. For example, following a {@link PutPortfolio}
 * from another AJO or at the start of an AJO's execution when the AJO has
 * streamed input files. Portfolios created by these means are initially
 * "hidden" so that their files are not in the Uspace. A hidden Portfolio is
 * revealed into the current Uspace directory when it is used by a task.
 * <p>
 * A Portfolio can be explicitly revealed  by inserting an instance of 
 * {@link DeclarePortfolio} into the AJO. Portfolios can be updated, e.g.
 * for each loop iteration, by forcing a reveal through a {@link DeclarePortfolio}
 * <p>
 * The execution of a {@link UserTask} can result in files being
 * created in the Uspace of which the NJS has no knowledge. These files
 * can be placed in a Portfolio and so be available to be manipulated
 * by other Tasks in the AJO by making them the subject of a
 * {@link MakePortfolio}.
 * <p>
 * Once Portfolios have been created they can be used by other tasks. 
 * Note that there is no mechanism during AJO construction to ensure that
 * the task to create a Portfolio executes before any tasks that use the
 * Portfolio. It is up to the creator of the AJO to ensure that the proper
 * dependancies are inserted between the tasks.
 *
 * @see org.unicore.Identifier
 * @see AbstractJob
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 1
 *
 * @version Portfolio.java 1.6 05/13/02 
 * 
 **/
public final class Portfolio extends org.unicore.Unicore {

    static final long serialVersionUID = -3607592189777665599L;

    public Portfolio() {
        this("");
    }

    /**
     * Create a new Portfolio of this name.
     *
     **/
    public Portfolio(String name) {
        id = new PortfolioIdentifier(name);
    }

    /**
     * Create a new Portfolio of this name and Identity.
     *
     **/
    public Portfolio(String name, int identity) {
        id = new PortfolioIdentifier(name, identity);
    }

    /**
     * Return the Portfolio's name. 
     * <p>
     * The name of a Portfolio is provided 
     * for the user's convenience, it has nothing to do
     * with the names of the files or with the identity
     * of the Portfolio.
     *
     **/
    public String getName() {return id.getName();}

    /**
     * Set the Portfolio's name.
     *
     **/
    public void setName(String name) {id.setName(name);}

    private PortfolioIdentifier id;
    /**
     * Return the Identifier of the Portfolio.
     * <p>
     * The Identifier is used to identify a Portfolio uniquely in
     * an AJO (Uspace). Portfolios need need an Identifier because they can be referred
     * to by AbstractActions in other AJOs that do not contain the
     * Portfolio. An example is the {@link GetPortfolio} task which fetches
     * a Portfolio from one Uspace into another Uspace. Portfolio Identifiers
     * are also used to link Portfolio contents transferred as streamed bytes
     * in the UPL to the Portfolios in the Uspace.
     *
     **/
    public PortfolioIdentifier getId() { return id; }

    /**
     * Test for equality using the Portfolio's Identifier. 
     * <p>
     * The Identifier is used to identify the Portfolio and so we
     * need to provide an implementation of equals that uses
     * this value.
     *
     **/
    public boolean equals(Object in) {
        return( in.getClass() ==  this.getClass() &&
                id.equals(((Portfolio)in).getId()) 
              );
    }

    /**
     * Return a hash code based on the Portfolio's Identifier.
     *
     **/
    public int hashCode() {return id.hashCode();}


    /**
     * Return a string for this Portfolio suitable for
     * naming directories in the UPL.
     *<p>
     * When streaming data the UPL places the files of a Portfolio 
     * in a directory of the data stream. This method ensures that
     * both ends of a transfer use the same name for the directory.
     *
     **/
    public String getUPLDirectoryName() {
        return "PF"+Integer.toHexString(id.getValue());
    }

    private String notes;
    /**
     * Return the notes added to this Portfolio.
     * <p>
     * The notes string is a space for clients to add information
     * about the purpose of an Portfolio.
     *
     **/
    public String getNotes() {return notes;}

    /**
     * Set some notes to this Portfolio.
     *
     **/
    public void setNotes(String notes) {this.notes = notes;}

    public String toString() {return getName();}

}
//
// Change log:
//          18 Mar 2002 SvdB V4 no arg constructor
//          30 Jan 2001 SvdB Added notes field
//          23 Nov 2000 SvdB Improved comments, added UPL directory
//          30 Apr 2000 SvdB edited for V3.0 
//           7 Dec 1998 SvdB Second release to Unicore 
//           7 Sep 1998 SvdB Added identifier 
//          12 Jun 1998 SvdB Initial release to Unicore 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
