package org.unicore.ajo;

/**
 * A RepeatGroup contains contains a DAG of actions that will be executed one
 * or more times, depending on the return code of a {@link Decidable} within the DAG.
 * <p>
 * The ActionGroup contained in a RepeatGroup will execute at least once. It
 * will execute within the same Uspace as the root AJO of the
 * RepeatGroup. Subsequent executions of the ActionGroup will also take place
 * within the same Uspace. Once all actions in the action group have completed
 * successfully, the {@link org.unicore.outcome.Decision} of the test task is
 * compared to the test value. If the strings are equal and the mode is UNTIL, then
 * RepeatGroup terminates with status SUCCESSFUL. If the strings are not equal
 * and the mode is WHILE, then RepeatGroup terminates with status SUCCESSFUL.
 * Otherwise, the NJS processes
 * the action group as if all statuses were again PENDING. The ActionGroup returns
 * to the READY status before returning to RUNNING for the next iteration.
 * <p>
 * If the ActionGorup does not complete successfully, then the iterations are
 * stopped and the RepeatGroup gets a final status of NOT_SUCCESSFUL. If the
 * Decision of the test task is null, then then the iterations are stopped
 * and the RepeatGroup gets a final status of NOT_SUCCESSFUL.
 * <p>
 * Note that all actions in one iteration
 * must be complete before the next iteration can start, e.g. this is not a parallel
 * construct.
 * <p>
 * The outcome of a RepeatGroup is a list of ActionGroup_Outcomes, one for each
 * iteration.
 * <p>
 * Precondition: The test task must be within the ActionGroup.
 * <p>
 * @see Decidable
 * @see org.unicore.outcome.RepeatGroup_Outcome
 * @see org.unicore.outcome.Decision
 *
 * @author D. Snelling (fecit)
 *
 * @since AJO 3.2
 *
 * @version RepeatGroup.java 1.8 09/09/02
 * 
 **/
public final class RepeatGroup extends ActionGroup {

    static final long serialVersionUID = -3727064362154807262L;

    /**
     * Create a new RepeatGroup.
     * <p>
     * @param name The name of the Repeat
     * @param extra_information Client supplied extra information about the RepeatGroup.
     * @param test_task The Decidable whose return code is used in the until expression.
     * @param test_value The string compared to the test task's Decision
     *
     **/
    public RepeatGroup( String name,
                        byte[] extra_information,
                        Decidable test_task,
                        String test_value
                      ) {
        super(name,extra_information);
        
        this.test_task  = test_task;
        this.test_value = test_value;
    }

    /**
     * Create a new RepeatGroup.
     * <p>
     * @param name The name of the Repeat
     * @param test_task The Decidable whose return code is used in the until expression.
     * @param test_value The string compared to the test task's Decision
     *
     **/
    public RepeatGroup( String name,
                        Decidable test_task,
                        String test_value
                      ) {
        this(name, (byte[]) null, test_task, test_value);
    }

    /**
     * Create a new RepeatGroup.
     * <p>
     * @param name The name of the Repeat
     *
     **/
    public RepeatGroup(String name) {
        this(name, (byte[]) null, (Decidable) null, (String) null);
    }

    public RepeatGroup() {
        this("");
    }

    private Decidable test_task;

    /**
     * Return the Decidable whose decision is to be tested.
     *
     **/
    public Decidable getTestTask() {return test_task;}

    /**
     * Set the test task.
     *
     **/
    public void setTestTask (Decidable test_task) {this.test_task = test_task;}

    private String test_value;
    /**
     * @return the test value
     *
     **/
    public String getTest() {return test_value;}

    /**
     * Set the test value.
     *
     **/
    public void setTest(String test_value) {this.test_value = test_value;}

    private boolean is_until = true;
    /**
     * Set the mode of the RepeatGroup to WHILE.
     * <p>
     * The RepeatGroup continues executing while the test task's Decision
     * is equal to the test value.
     *
     **/
    public void setWhile() {is_until = false;}

    /**
     * Set the mode of the RepeatGroup to UNTIL.
     * <p>
     * The RepeatGroup continues executing until the test task's Decision
     * is equal to the test value.
     *
     **/
    public void setUntil() {is_until = true;}

    public boolean isUntil() {return is_until;}    

}
//
//  Change history:
//        8 Aug 2002 SvdB Uses Decidable
//       18 Mar 2002 SvdB V4 no arg constructor
//       31 Jan 2001  DFS Created
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
