package org.unicore.ajo;

import org.unicore.sets.ResourceSet;

/**
 * Copy the files making up a Portfolio to the spool area.
 * <p>
 * A Uspace is deleted at the end of an AbstractJob's execution
 * and so all files created during the execution of the AJO are 
 * lost unless there are AbstractActions within the AJO that
 * explicitly save the files to other locations. Files can
 * be exported (using {@link ExportTask}), copied to the AJO's
 * Outcome (using {@link CopyPortfolioTask}) or copied to the spool area
 * using this AbstactAction.
 * <p>
 * The spool area is some file space within a Vsite known to
 * the NJS that can be used for semi-permanent storage of
 * files (as Portfolios). Files that have been spooled can 
 * be copied into the Uspace of another AJO (using {@link UnSpool}
 * or {@link CopySpooled}).
 * Unspooling deletes the Portfolio from the spool area copying
 * does not (spooled Portfolios can also be deleted using 
 * {@link DeleteSpooled}).
 * <p>
 * An example of the use of the spool area is for AJOs that
 * produce large files. A user may not wish to have these files
 * automatically returned to her workstation, as happens with
 * CopyPortfolioTask, but would rather retrieve them when she
 * knows that the AJO executed correctly. This can be done
 * by spooling the Portfolio during execution of the main AJO
 * and then consigning further AJOs to unspool the Portfolio
 * and return the files.
 * <p>
 * Spooling is a limited form of export, in particular the permissions
 * on spooled files are set by the Xlogin used for the Spool task.
 * The Xlogins of tasks using spooled files must have 
 * permission to access these files or these tasks will fail.
 * 
 * @deprecated AJO 4.0 Use {@link PutPortfolio} with the {@link org.unicore.resources.Spool} resource.
 *
 * @see Portfolio
 * @see org.unicore.outcome.Spool_Outcome
 * @see UnSpool
 * @see DeleteSpooled
 * @see CopySpooled
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3.2
 *
 * @version Spool.java 1.5 05/13/02
 * 
 **/
public final class Spool extends FileTask {

    static final long serialVersionUID = 6668036600849945101L;

    /**
     * Create a new task to spool the Portfolio that
     * requests the given resources and saves the 
     * supplied extra information.
     *
     * @param name The name of the task.
     * @param portfolio The Portfolio to spool
     * @param extra_information Storage for data from the creator of this task.
     * @param resources The resources requested by this task.
     *
     **/
    public Spool( String name,
                  Portfolio portfolio,
                  byte[] extra_information,
                  ResourceSet resources
                ) {
        super(name,extra_information,resources);
        this.portfolio = portfolio;
    }

    /**
     * Create a new task to spool the Portfolio that
     * requests the given resources.
     *
     * @param name The name of the task.
     * @param portfolio The Portfolio to spool
     * @param resources The resources requested by this task.
     *
     **/
    public Spool ( String name,
                   Portfolio portfolio,
                   ResourceSet resources
                 ) {

        this(name,portfolio, (byte[]) null,resources);
    }

    /**
     * Create a new Spool with this name and
     * empty values otherwise.
     *
     **/
    public Spool(String name) {
        this(name, (Portfolio) null, (byte[]) null, (ResourceSet) null);
    }

    public Spool() {
        this("");
    }

    private Portfolio portfolio;
    /**
     * Return the Portfolio to be spooled by this task.
     *
     **/
    public Portfolio getPortfolio() {
        return portfolio;
    }

    /**
     * Set the Portfolio to be spooled by this task.
     **/
    public void setPortfolio(Portfolio portfolio) {this.portfolio = portfolio;}

}
//
// Change history:
//          18 Mar 2002 SvdB V4 no arg constructor, deprecated
//          30 Jan 2001 SvdB Created. 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
