package org.unicore.outcome;

import org.unicore.ajo.AbstractJob;

import java.util.Hashtable;

/**
 * The results of executing an {@link org.unicore.ajo.AbstractJob}. 
 * <p>
 * Instances of this class are handled by the UPL (in {@link org.unicore.upl.RetrieveOutcomeReply})
 * and are passed between NJSs. The NJS consigning a child AJO to another NJS
 * will hold the child AJO's AbstractJob_Outcome in the parent's AJO Outcome.
 * If the childs AJO's AbstractJob_Outcome is fully deserialised at the 
 * parent's NJS, then to avoid deserialisation problems we would be forced to ensure that every NJS in a Unicore
 * system was always using the same version of the AJO classes. To avoid
 * this undesirable situation, AbstractJob_Outcome holds the constituent
 * Outcomes serialised to a byte array. This means that before being used by a client the 
 * AbstractJob_Outcome must be completely deserialised by passsing it
 * as the argument to {@link org.unicore.utility.ConsignForm#convertFrom(AbstractJob_Outcome)}.
 * <p>
 * @see org.unicore.ajo.AbstractJob
 * @see org.unicore.utility.ConsignForm
 * @see org.unicore.upl.RetrieveOutcomeReply
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 2
 *
 * @version AbstractJob_Outcome.java 1.8 05/13/02
 * 
 **/
public class AbstractJob_Outcome extends ActionGroup_Outcome {

    static final long serialVersionUID = 7763375949171027834L;

    public AbstractJob_Outcome() {
        this(null, null);
    }

    /**
     * Create a new AbstractJob_Outcome.
     * <p>
     * This will hold the standard information from {@link Outcome}
     * plus the Outcomes of all child AbstractActions as in {@link ActionGroup_Outcome}
     *
     * @param source The AbstractJob  for which this Outcome will hold the results.
     * @param initial The status of the AbstractJob at the time that the Outcome is created.
     *
     **/
    public AbstractJob_Outcome(AbstractJob source, AbstractActionStatus initial) {
        super(source, initial);
    }

    private byte[] anon_outcomes;
    /**
     * Get the anonymous bytes of the Outcome as a byte array. 
     * <p> 
     * This the form in which Outcomes are passed to parent AJOs so that
     * they do not have to be deserialised (as these Outcomes could
     * contain classes that are not known to intermediate NJSs). 
     * <p>
     * Methods to perform the conversion to and from this anonymous form
     * are provided in {@link org.unicore.utility.ConsignForm}.
     *
     **/
    public byte[] getAnonOutcomes() {return anon_outcomes;}

    /**
     * Set the anonymous (serialised) form of the Outcome.
     * <p>
     * This also clears the non-anonymous version.
     *
     **/
    public void setAnonOutcomes(byte[] anon_outcomes) {
        this.anon_outcomes = anon_outcomes;
        child_outcomes = new Hashtable(11);
    }
}
//
// Change history:
//          18 Mar 2002 SvdB V4, no arg constructor
//          26 Jun 2000 SvdB Added anonymous bytes for transfer of Outcomes
//           5 May 2000 SvdB Created
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
