package org.unicore.outcome;

import org.unicore.Unicore;

import java.util.*;

/**
 * The history of status changes for an AbstractAction
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 4.0
 *
 * @version StatusHistory.java 1.5 07/04/02 
 * 
 **/
public class StatusHistory extends Unicore {

    static final long serialVersionUID = 0;

    private SortedSet history;

    /**
     * Create a new history.
     *
     **/
    public StatusHistory() {
        history = new TreeSet(new StatusHistoryComparator());
    }

    private class StatusHistoryComparator extends Unicore implements Comparator {
        public int compare(Object o1, Object o2) {
            if(o1 instanceof StatusHistory.Entry && o2 instanceof StatusHistory.Entry) {
                return ((StatusHistory.Entry)o1).getTime().compareTo( ((StatusHistory.Entry)o2).getTime());
            }
            else {
                throw new ClassCastException("org.unicore.outcome.StatusHistory. Not right types "+o1+" "+o2);
            }
        }
    }

    private AbstractActionStatus current_status;
    /**
     * Get the current status for the AbstractAction
     *
     **/
    public final AbstractActionStatus getCurrentStatus() {return current_status;}

    private String last_reason;
    /**
     * Get the last added reason
     *
     **/
    public final String getLastReason() {return last_reason;}

    private Date last_time;
    public final Date getLastTime() {return last_time;}

    /**
     * Add an entry to the history.
     *
     * The entry gets the current time, also updates the current status and reason
     *
     * @param status The current new status
     * @param reason The new latest reason
     *
     **/
    public void add(AbstractActionStatus status, String reason) {
        current_status = status;
        last_reason = (reason == null?"":reason);
        last_time = new Date();
        history.add(new StatusHistory.Entry(status, last_reason, last_time));
    }

    /**
     * Returns an iteration over the history (in time order).
     *
     * The returned Objects are instances of StatusHistory.Entry
     *
     **/
    public Iterator iterator() {
        return history.iterator();
    }

    /**
     * Merge the history into this one.
     *
     * Combines the two histories in time order
     *
     * Leaves the curremt status and reason unchanged
     *
     **/
    public void merge(StatusHistory input) {
        try {
            Iterator i = input.iterator();
            while(i.hasNext()) {
                this.history.add(i.next());
            }
        }
        catch(Exception ex) {
        }
    }

    /**
     * A single entry in a status history
     *
     **/
    public static class Entry extends Unicore {

        public Entry(AbstractActionStatus status, String reason, Date time) {
            this.status = status;
            this.reason = reason;
            this.time = time;
        }

        private AbstractActionStatus status;
        public AbstractActionStatus getStatus() {return status;}

        private String reason;
        public String getReason() {return reason;}

        private Date time;
        public Date getTime() {return time;}

        public boolean equals(Object o) {
            boolean ress = false;
            boolean rest = false;
            if(o instanceof StatusHistory.Entry) {
                StatusHistory.Entry test = (StatusHistory.Entry)o;
                if(status != null) ress = status.equals(test.status);
                if(time   != null) rest = time.equals(test.time);
            }
            return ress && rest;
        }

        public int hashCode() {return time.hashCode();}
    }

}
//
// Change history:
//          19 Jun 2002 SvdB Need explicit comparator class to make it serialisable
//                           added equals method to Entry
//          11 Apr 2002 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
